package readxls;
/*-**********************************************************************
 * File: ReadXls.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.List;

import readxls.control.Breaker;
import readxls.control.CellDefine;
import readxls.control.DetailCallable;
import readxls.control.DetailDefine;
import readxls.control.HeaderCallable;
import readxls.control.HeaderDefine;
import readxls.control.LocaleInfo;
import readxls.control.RowColumnBreaker;
import readxls.type.CellTypeDefine;
import readxlsforjxl.XlsHandleForJxl;
import readxlsforpoi.XlsHandleForPoi;

/** Excel ファイルを読み込むパッケージ.
 * <P>
 * Excel ファイルをデータ入力用のファイル形式として使用するための機能を
 * 実装している。
 * </P>
 * <P>
 * 使用方法は、
 * </P>
 * <OL>
 * <LI>
 * ヘッダの定義:
 * <BR>
 * HeaderDefine のリストに、ヘッダ情報を構成するセルの定義情報を設定する。
 * </LI>
 * <LI>
 * ヘッダ情報すべてを読み込んだ際のコールバックを指定:
 * <BR>
 * HeaderCallable を実装したクラスのインスタンスを指定する。
 * コールバックが不用であれば、null を指定する。
 * </LI>
 * <LI>
 * 明細の定義:
 * <BR>
 * DetailDefine のリストに、明細を構成する各行の定義情報を設定する。
 * <BR>
 * 設定した DetailDefine のリストのリストに、それらを設定する。
 * </LI>
 * <LI>
 * 明細情報一件分を読み込んだ際のコールバックを指定:
 * <BR>
 * DetailCallable を実装したクラスのインスタンスを指定する。
 * コールバックが不用であれば、null を指定する。
 * </LI>
 * <LI>
 * 明細情報の読み込み停止制御情報の定義:
 * <BR>
 * 行・列の読み込み停止条件を RowColumnBreaker に設定する。
 * </LI>
 * <LI>
 * ヘッダ情報ハッシュテーブル Hashtable&lt;String, String&gt; の生成。
 * <BR>
 * このハッシュテーブルから項目名称をキーで、
 * ヘッダ項目の入力値を得られる。
 * </LI>
 * <LI>
 * 明細情報リスト List&lt;List&lt;String&gt;&gt; の生成。
 * <BR>
 * このリストは、明細の各行の入力値のリストを
 * 読み込んだ行の順に並べたリストになる。
 * </LI>
 * <LI>
 * 不正データ情報リスト List&lt;InvalidCellInfo&gt; の生成。
 * <BR>
 * このリストは、読み込み中に入力値の不正を検知した
 * セルの情報のリストになる。
 * </LI>
 * <LI>
 * 次のいずれかの Excel ファイル読み込み手順を呼び出す:
 * <UL>
 * <LI>
 * {@link ReadXls#readInputStream(InputStream, LocaleInfo, XlsHandler, boolean, String, List, HeaderCallable, boolean, int, Breaker, List, DetailCallable, int, Hashtable, List, List, InvalidCellInfoCollector)}
 * で一度のメソッド呼び出しでヘッダと明細を読み込む。
 * </LI>
 * <LI>
 * {@link ReadXls#createXlsReader(InputStream, XlsHandler, boolean)}
 * か
 * {@link ReadXls#createXlsReader(InputStream, LocaleInfo, XlsHandler, boolean)}
 * で ReadXls インスタンスを生成し、
 * <BR>
 * {@link ReadXls#parse(Hashtable, List, List)}
 * でヘッダと明細それぞれを読み込む。
 * <BR>
 * 読み込みがすべてすんだら、必ず、
 * {@link ReadXls#close()}
 * で ReadXls インスタンスに保持しているリソースを解放する。
 * </LI>
 * </UL>
 * <LI>
 * readInputStream() あるいは parse() の戻り値が
 * {@link ReturnCode#NORMAL}
 * でなければ、読み込みエラーに応じた処理を行う。
 * <BR>
 * {@link ReturnCode#FOUND_INVALID_CELLS}
 * が返された場合、不正データ情報リスト List&lt;InvalidCellInfo&gt;
 * から不正な入力がされているセルの情報を取得する。
 * </LI>
 * <LI>
 * ヘッダの各項目の入力値は、ヘッダ情報ハッシュテーブル
 * Hashtable&lt;String, String&gt; から項目名称をキーにして取得する。
 * </LI>
 * <LI>
 * 明細の各行は、明細情報リスト List&lt;List&lt;String&gt;&gt;
 * の要素 List&lt;String&gt; として取得する。各行の列の入力値は、
 * List&lt;String&gt; から取得する。
 * <BR>
 * ある行番号のある列番号の値を求めるのは以下のようになる。
 * <CODE>入力値 = 明細情報リスト.get(行番号 - 1).get(列番号 - 1)</CODE>
 * </LI>
 * </OL>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class ReadXls implements InvalidCellInfoCollector {

	/** 定数.
	 * <P>
	 * 空セルの値 ("")
	 * </P>
	 */
	public static final String EMPTY_VALUE = "";

	/** リソース.
	 * <P>
	 * Excel ファイルの入力元ストリーム。
	 * </P>
	 */
	private InputStream is = null;

	/** リソース.
	 * <P>
	 * ロケール情報。
	 * </P>
	 */
	private LocaleInfo localeInfo = null;

	/** リソース.
	 * <P>
	 * 依存ライブラリを使用する Excel ファイル読み込みハンドラ。
	 * </P>
	 */
	private XlsHandler xlsHandler = null;

	/** リソース.
	 * <P>
	 * Excel ファイルが Office Open XML 形式 (Excel2007 以降)
	 * であるか否かを示す。
	 * </P>
	 */
	private boolean isOOXML = false;

	/** リソース.
	 * <P>
	 * 不正データ情報を出力するリスト。
	 * </P>
	 */
	private List<InvalidCellInfo> invalidCellInfos = null;

	/** リソース.
	 * <P>
	 * 不正データ情報を検知した数。
	 * </P>
	 */
	private int foundInvalidCellNum = 0;

	/** リソース.
	 * <P>
	 * 不正データを検知する最大数。
	 * <BR>
	 * 検知した不正でー多数がこの最大数になったら読み込み処理は中断する。
	 * </P>
	 */
	private int maxInvalidCellNum = 0;

	/** リソース.
	 * <P>
	 * 不正データを検知した際に、そのセルの不正データ情報を通知する先。
	 * </P>
	 */
	private InvalidCellInfoCollector invalidCellInfoCollector = null;

	/** リソース.
	 * <P>
	 * シートから読み込める行の行数。
	 * </P>
	 */
	private int readableRowNum = 0;

	/** リソース.
	 * <P>
	 * 最後に読み込んだ行の行番号 (1～)
	 * </P>
	 */
	private int lastReadRowNo = 0;

	/** リソース.
	 * <P>
	 * 読み込んだヘッダ情報のコールバック
	 * </P>
	 */
	private HeaderCallable headerCallable = null;

	/** リソース.
	 * <P>
	 * 一件分の明細情報を読み込むたびに呼び出すコールバック
	 * </P>
	 */
	private DetailCallable detailCallable = null;

	/** 属性.
	 * <P>
	 * 読み込むシートの名称。
	 * </P>
	 */
	private String sheetName = null;

	/** 属性.
	 * <P>
	 * ヘッダ情報を読み込むセルそれぞれの項目定義
	 * {@link HeaderDefine}
	 * のリスト。
	 * </P>
	 */
	private List<HeaderDefine> headerDefines = null;

	/** 属性.
	 * <P>
	 * ヘッダ情報を読み込んだ際に不正データが検知された場合に、
	 * 読み込み処理を停止するならば true
	 * を、明細部の読み込みを行うならば false を指定する。
	 * </P>
	 */
	private boolean stopWhenHeaderHasInvalidValue = false;

	/** 属性.
	 * <P>
	 * 明細行を読み込む歳帆の行番号 (1～)。
	 * </P>
	 */
	private int firstDetailRowNo = 0;

	/** 属性.
	 * <P>
	 * 行・列の読み込みを制御する {@link Breaker} 。
	 * </P>
	 */
	private Breaker breaker = null;

	/** 属性.
	 * <P>
	 * 明細を構成する行それぞれの項目定義
	 * {@link DetailDefine}
	 * のリスト。
	 * </P>
	 */
	private List<DetailDefine> detailDefines = null;

	/** gettar/setter.
	 * @return
	 * {@link ReadXls#is}
	 * を返す。
	 */
	private InputStream getIs() {
		return this.is;
	}

	/** getter/setter.
	 * @param is
	 * {@link ReadXls#is}
	 * に指定する Excel ファイルの入力元ストリーム。
	 */
	private void setIs(final InputStream is) {
		this.is = is;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#localeInfo}
	 * を返す。
	 */
	public final LocaleInfo getLocaleInfo() {
		return this.localeInfo;
	}

	/** getter/setter.
	 * @param localeInfo
	 * {@link ReadXls#localeInfo}
	 * に設定するロケール情報。
	 */
	private void setLocaleInfo(final LocaleInfo localeInfo) {
		this.localeInfo = localeInfo;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#xlsHandler}
	 * を返す。
	 */
	private XlsHandler getXlsHandler() {
		return this.xlsHandler;
	}

	/** getter/setter.
	 * @param xlsHandler
	 * {@link ReadXls#xlsHandler}
	 * に設定する依存ライブラリを使用して Excel ファイル読み込みハンドル。
	 */
	private void setXlsHandler(final XlsHandler xlsHandler) {
		this.xlsHandler = xlsHandler;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#isOOXML}
	 * を返す。
	 */
	private boolean isOOXML() {
		return isOOXML;
	}

	/** getter/setter.
	 * @param isOOXML
	 * {@link ReadXls#isOOXML}
	 * に設定する値。
	 */
	private void setOOXML(final boolean isOOXML) {
		this.isOOXML = isOOXML;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#invalidCellInfos}
	 * を返す。
	 */
	private List<InvalidCellInfo> getInvalidCellInfos() {
		return this.invalidCellInfos;
	}

	/** getter/setter.
	 * @param invalidCellInfos
	 * {@link ReadXls#invalidCellInfos}
	 * に設定する {@link InvalidCellInfo} のリスト。
	 */
	private void setInvalidCellInfos(
							final List<InvalidCellInfo> invalidCellInfos) {
		this.invalidCellInfos = invalidCellInfos;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#foundInvalidCellNum}
	 * を返す。
	 */
	private int getFoundInvalidCellNum() {
		return this.foundInvalidCellNum;
	}

	/** getter/setter.
	 * @param foundInvalidCellNum
	 * {@link ReadXls#foundInvalidCellNum}
	 * に設定する値 (0～)。
	 */
	private void setFoundInvalidCellNum(final int foundInvalidCellNum) {
		this.foundInvalidCellNum = foundInvalidCellNum;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#maxInvalidCellNum}
	 * を返す。
	 */
	public final int getMaxInvalidCellNum() {
		return this.maxInvalidCellNum;
	}

	/** getter/setter.
	 * @param maxInvalidCellNum
	 * {@link ReadXls#maxInvalidCellNum}
	 * に設定する値 (1～)。
	 */
	public final void setMaxInvalidCellNum(final int maxInvalidCellNum) {
		this.maxInvalidCellNum = maxInvalidCellNum;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#invalidCellInfoCollector}
	 * を返す。
	 * @see InvalidCellInfoCollector
	 */
	public final InvalidCellInfoCollector getInvalidCellInfoCollector() {
		return this.invalidCellInfoCollector;
	}

	/** getter/setter.
	 * @param invalidCellInfoCollector
	 * {@link ReadXls#invalidCellInfoCollector}
	 * に設定する不正データ検知通知受信インスタンス。
	 * @see InvalidCellInfoCollector
	 */
	public final void setInvalidCellInfoCollector(
				final InvalidCellInfoCollector invalidCellInfoCollector) {
		this.invalidCellInfoCollector = invalidCellInfoCollector;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#readableRowNum}
	 * を返す。
	 */
	private int getReadableRowNum() {
		return this.readableRowNum;
	}

	/** getter/setter.
	 * @param readableRowNum
	 * {@link ReadXls#readableRowNum}
	 * に設定する値 (0～)。
	 */
	private void setReadableRowNum(final int readableRowNum) {
		this.readableRowNum = readableRowNum;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#lastReadRowNo}
	 * を返す。
	 */
	private int getLastReadRowNo() {
		return this.lastReadRowNo;
	}

	/** getter/setter.
	 * @param lastReadRowNo
	 * {@link ReadXls#lastReadRowNo}
	 * に設定する値 (1～)。
	 */
	private void setLastReadRowNo(final int lastReadRowNo) {
		this.lastReadRowNo = lastReadRowNo;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#headerCallable} を返す。
	 */
	public final HeaderCallable getHeaderCallable() {
		return headerCallable;
	}

	/** getter/setter.
	 * @param headerCallable
	 * {@link ReadXls#headerCallable} に設定する {@link HeaderCallable}
	 */
	public final void setHeaderCallable(
									final HeaderCallable headerCallable) {
		this.headerCallable = headerCallable;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#detailCallable} を返す。
	 */
	public final DetailCallable getDetailCallable() {
		return detailCallable;
	}

	/** getter/setter.
	 * @param detailCallable
	 * {@link ReadXls#detailCallable} に設定する {@link DetailCallable}
	 */
	public final void setDetailCallable(final DetailCallable detailCallable) {
		this.detailCallable = detailCallable;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#sheetName} を返す。
	 */
	public final String getSheetName() {
		return sheetName;
	}

	/** getter/setter.
	 * @param sheetName
	 * {@link ReadXls#sheetName} に設定するシート名。
	 */
	public final void setSheetName(final String sheetName) {
		this.sheetName = sheetName;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#headerDefines} を返す。
	 */
	public final List<HeaderDefine> getHeaderDefines() {
		return headerDefines;
	}

	/** getter/setter.
	 * @param headerDefines
	 * {@link ReadXls#headerDefines} に設定する
	 * {@link HeaderDefine} のリスト。
	 */
	public final void setHeaderDefines(
								final List<HeaderDefine> headerDefines) {
		this.headerDefines = headerDefines;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#stopWhenHeaderHasInvalidValue} を返す。
	 */
	public final boolean isStopWhenHeaderHasInvalidValue() {
		return stopWhenHeaderHasInvalidValue;
	}

	/** getter/setter.
	 * @param stopWhenHeaderHasInvalidValue
	 * {@link ReadXls#stopWhenHeaderHasInvalidValue} に設定する論理値。
	 */
	public final void setStopWhenHeaderHasInvalidValue(
							final boolean stopWhenHeaderHasInvalidValue) {
		this.stopWhenHeaderHasInvalidValue = stopWhenHeaderHasInvalidValue;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#firstDetailRowNo} を返す。
	 */
	public final int getFirstDetailRowNo() {
		return firstDetailRowNo;
	}

	/** getter/setter.
	 * @param firstDetailRowNo
	 * {@link ReadXls#firstDetailRowNo} に設定する行番号 (1～)。
	 */
	public final void setFirstDetailRowNo(final int firstDetailRowNo) {
		this.firstDetailRowNo = firstDetailRowNo;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#breaker} を返す。
	 */
	public final Breaker getBreaker() {
		return breaker;
	}

	/** getter/setter.
	 * @param breaker
	 * {@link ReadXls#breaker} に設定する {@link Breaker} 。
	 */
	public final void setBreaker(final Breaker breaker) {
		this.breaker = breaker;
	}

	/** getter/setter.
	 * @return
	 * {@link ReadXls#detailDefines} を返す。
	 */
	public final List<DetailDefine> getDetailDefines() {
		return detailDefines;
	}

	/** getter/setter.
	 * @param detailDefines
	 * {@link ReadXls#detailDefines} に設定する
	 * {@link DetailDefine} のリスト。
	 */
	public final void setDetailDefines(
								final List<DetailDefine> detailDefines) {
		this.detailDefines = detailDefines;
	}

	/** ユーティリティ.
	 * <P>
	 * ヘッダ情報定義 List&lt;{@link HeaderDefine}&gt;
	 * に設定されている定義それぞれが正当であることを
	 * {@link HeaderDefine#isValid(XlsHandler, LocaleInfo)}
	 * で確認する。
	 * </P>
	 * @return
	 * 設定されているヘッダ情報定義がすべて正当なものであれば true
	 * を、そうでなければ false を返す。
	 */
	private boolean isValidHeaderDefines() {
		if (getHeaderDefines() != null) {
			for (HeaderDefine hd : getHeaderDefines()) {
				if (!hd.isValid(getXlsHandler(), getLocaleInfo())) {
					return false;
				}
			}
		}
		return true;
	}

	/** ユーティリティ.
	 * <P>
	 * 明細定義情報 List&lt;{@link DetailDefine}&gt;
	 * に設定されているすべての定義が正当なものであることを、確認する。
	 * </P>
	 * <P>
	 * 行・列の読み込み停止位置を求める {@link Breaker}
	 * が指定されていれば、その設定が正当であることも確認する。
	 * </P>
	 * @return
	 * 明細定義情報が正当であれば true を、そうでなければ false を返す。
	 * @see RowColumnBreaker
	 */
	private boolean isValidDetailDefines() {
		if (getDetailDefines() != null) {
			if (getFirstDetailRowNo() < 1) {
				return false;
			}
			if (getBreaker() != null && !getBreaker().isValid()) {
				return false;
			}
			for (DetailDefine dd : getDetailDefines()) {
				if (!dd.isValid(getXlsHandler(), getLocaleInfo())) {
					return false;
				}
			}
		}
		return true;
	}

	/** ユーティリティ.
	 * <P>
	 * 指定された行番号 (1～) と列番号 (1～) のセルを読み込む。
	 * <BR>
	 * 読み込みを停止する行を超えた行番号が指定されていれば、
	 * 要素がゼロの行を読み込んだ上で、セルは null とする。
	 * <BR>
	 * 最後にこのメソッドを実行した際の行番号と同じ行が
	 * 指定されていれば、行内のセルの読み込みは行わない。
	 * </P>
	 * <P>
	 * 指定された列番号が、読み込みを停止する列番号を超えていれば、
	 * セルは null とする。
	 * </P>
	 * @param rowNo
	 * 行番号 (1～)
	 * @param columnNo
	 * 列番号 (1～)
	 * @param breakRowNo
	 * 読み込みを停止する行番号 (1～)
	 * @param breakCokumnNo
	 * 読み込みを停止する列番号 (1～)
	 */
	private void getCell(final int 		rowNo,
							final int	columnNo,
							final int	breakRowNo,
							final int	breakCokumnNo) {
		if (breakRowNo < rowNo) {
			getXlsHandler().readInCells(0);
		}
		else if (getLastReadRowNo() != rowNo) {
			getXlsHandler().readInCells(rowNo);
		}
		setLastReadRowNo(rowNo);
		getXlsHandler().readInLastReadCell(
				(breakCokumnNo >= columnNo
				&& getXlsHandler().lengthOfLastReadCells() >= columnNo)
				?	columnNo
				:	0);
	}

	/** ユーティリティ.
	 * <P>
	 * {@link ReadXls#getCell(int, int, int, int)}
	 * で、読み込む行・列のセルを読み込み、
	 * {@link CellDefine#readCell(int, int, XlsHandler, InvalidCellInfoCollector)}
	 * で、そのセルから入力値を取得する。
	 * <BR>
	 * 指定されたセルが読み込み可能な行列範囲から外れた位置か未入力ならば、
	 * 空文字列 "" を入力値とする。
	 * </P>
	 * @param cd
	 * セル定義情報 {@link CellDefine}
	 * @param rowNo
	 * 行番号 (1～)
	 * @param columnNo
	 * 列番号 (1～)
	 * @param breakRowNo
	 * 読み込みを停止する行番号 (1～)
	 * @param breakCokumnNo
	 * 読み込みを停止する列番号 (1～)
	 * @return
	 * セルの入力値。
	 * @throws ReadXlsException
	 * セルが不正なものであれば発生する。
	 */
	private String getValue(final CellDefine	cd,
							final int			rowNo,
							final int			columnNo,
							final int			breakRowNo,
							final int			breakCokumnNo)
									throws ReadXlsException {
		getCell(rowNo, columnNo, breakRowNo, breakCokumnNo);
		String value = cd.readCell(rowNo, columnNo, getXlsHandler(), this);
		if (value == null) {
			value = ReadXls.EMPTY_VALUE;
		}
		return value;
	}

	/** ユーティリティ.
	 * <P>
	 * {@link ReadXls#getValue(CellDefine, int, int, int, int)}
	 * で、ヘッダ部のセルの入力値を取得する。
	 * </P>
	 * @param cd
	 * セル定義情報 {@link CellDefine}
	 * @param rowNo
	 * 行番号 (1～)
	 * @param columnNo
	 * 列番号 (1～)
	 * @return
	 * セルの入力値
	 * @throws ReadXlsException
	 * セルが不正なものであれば発生する。
	 */
	private String getValueForHeader(final CellDefine	cd,
										final int		rowNo,
										final int		columnNo)
											throws ReadXlsException {
		return getValue(cd,
						rowNo,
						columnNo,
						getReadableRowNum(),
						Integer.MAX_VALUE);
	}

	/** ユーティリティ.
	 * <P>
	 * {@link ReadXls#foundInvalidCellNum}
	 * が
	 * {@link ReadXls#maxInvalidCellNum}
	 * 以上になっていれば、リターンコード
	 * {@link ReturnCode#FOUND_INVALID_CELLS}
	 * の {@link ReadXlsException} を発生する。
	 * </P>
	 * @throws ReadXlsException
	 * 不正データの検知数が上限に到達すれば発生する。
	 */
	private void checkFoundInvalidCellNum() throws ReadXlsException {
		if (getFoundInvalidCellNum() >= getMaxInvalidCellNum()) {
			throw new ReadXlsException(ReturnCode.FOUND_INVALID_CELLS);
		}
	}

	/** ユーティリティ.
	 * <P>
	 * ヘッダ部のセルすべての入力値を取得する。
	 * </P>
	 * @param header
	 * ヘッダ部のすべてのセルの入力値を収める
	 * Hashtavle&lt;String,String&gt;。
	 * キーは headerDefines で指定した項目名称。
	 * @throws ReadXlsException
	 * セルが不正なものであれば発生する。
	 */
	private void readHeaders(final Hashtable<String, String> header)
												throws ReadXlsException {
		for (HeaderDefine hd : getHeaderDefines()) {
			CellDefine cd = hd.getCellDefine();
			String value = getValueForHeader(cd,
											hd.getRowNo(),
											cd.getColumnNo());
			header.put(cd.getName(), value);
			checkFoundInvalidCellNum();
		}
	}

	/** ユーティリティ.
	 * <P>
	 * 指定された列範囲を埋めるだけの空セルの入力値
	 * {@link ReadXls#EMPTY_VALUE}
	 * を挿入する。
	 * </P>
	 * @param cno
	 * 列範囲の開始列番号 (1～)
	 * @param nextCno
	 * 列範囲の終了列番号 (1～)
	 * @param values
	 * 空セルの入力値を挿入する行の入力値出力先リスト
	 */
	private void fillEmptyCell(final int			cno,
								final int			nextCno,
								final List<String>	values) {
		int limit = nextCno - cno;
		for (int i = 0; i < limit; ++i) {
			values.add(ReadXls.EMPTY_VALUE);
		}
	}

	/** ユーティリティ.
	 * <P>
	 * 開始列から終了列の左隣までのセルの入力値を、明細情報に追加する。
	 * <BR>
	 * 明細情報にすでに保持している列と開始列の間に間隔があいているならば、
	 * 空セルを明細情報に挿入する。
	 * </P>
	 * @param cds
	 * 明細情報を構成する行数分の CellDefine のリスト
	 * @param startIdx
	 * 開始列の列番号 (1～)
	 * @param limitIdx
	 * 終了列の列番号 (1～)
	 * @param rno
	 * 行番号 (1～)
	 * @param cno
	 * すでに明細情報に保持している列の列番号 (1～)
	 * @param values
	 * 名細部から読み込んだ入力値 String のリスト
	 * @param breakRowNo
	 * 読み込みを停止する行の行番号 (1～)
	 * @param breakCokumnNo
	 * 読み込みを停止する列の列番号 (1～)
	 * @return
	 * 最後に読み込んだ列の列番号 (1～)
	 * @throws ReadXlsException
	 * セルが不正なものであれば発生する。
	 */
	private int addValue(final List<CellDefine>	cds,
							final int			startIdx,
							final int			limitIdx,
							final int			rno,
							final int			cno,
							final List<String>	values,
							final int			breakRowNo,
							final int			breakCokumnNo)
								throws ReadXlsException {
		int retCno = cno;
		for (int i = startIdx; i < limitIdx; ++i) {
			CellDefine cd = cds.get(i);
			int readCno = cno + cd.getColumnNo()
								- cds.get(startIdx).getColumnNo();
			fillEmptyCell(retCno, readCno, values);
			values.add(getValue(cd,
								rno,
								readCno,
								breakRowNo,
								breakCokumnNo));
			retCno = readCno + 1;
			checkFoundInvalidCellNum();
		}
		return retCno;
	}

	/** ユーティリティ.
	 * <P>
	 * 明細情報を読み込む。
	 * </P>
	 * <OL>
	 * <LI>
	 * 読み込みを停止する行の行番号を求める。
	 * <BR>
	 * シートに入力されていない行から明細情報を求める分については、
	 * すべての列に空セルが入力されている行から読み込んだものと同等になる
	 * ように補正する。
	 * </LI>
	 * <LI>
	 * 読み込みを停止する列の行番号を求める。
	 * </LI>
	 * <LI>
	 * 繰り返し読み込みを行わない列の部分を読み込む。
	 * </LI>
	 * <LI>
	 * 繰り返し読み込みを行う列の部分を読み込む。
	 * <BR>
	 * 読み込みを停止する列を超えている列番号のセルからは、
	 * 空セルを読み込んだものとする。
	 * </LI>
	 * <LI>
	 * 読み込んだセルの内容は、明細情報の行数分の
	 * String リストに設定する。
	 * </LI>
	 * </OL>
	 * @param details
	 * 明細情報から読み込んだ入力値を設定する String のリスト
	 * @throws ReadXlsException
	 * セルが不正なものであれば発生する。
	 */
	private void readDetails(final List<List<String>> details)
												throws ReadXlsException {
		int maxRowNo = 0;
		int breakRowNo = (getBreaker() != null)
						?	getBreaker().getBreakRowNo(getXlsHandler(),
												firstDetailRowNo,
												getReadableRowNum())
						:	getReadableRowNum();
		if (breakRowNo >= getFirstDetailRowNo()) {
			maxRowNo = (((breakRowNo - getFirstDetailRowNo())
							/ getDetailDefines().size()) + 1)
							* getDetailDefines().size();
		}
		maxRowNo += (getFirstDetailRowNo() - 1);
		int breakColumnNo = 0;
		List<List<String>>	detailBlock = new LinkedList<List<String>>();
		for (int rno = getFirstDetailRowNo(); rno <= maxRowNo; ++rno) {
			// read in the row cells.
			getCell(rno, 1, breakRowNo, Integer.MAX_VALUE);
			List<String> values = new LinkedList<String>();
			int ddIdx = (rno - getFirstDetailRowNo())
											% getDetailDefines().size();
			if (getBreaker() == null) {
				breakColumnNo = getXlsHandler().lengthOfLastReadCells();
			} else if (ddIdx == 0) {
				breakColumnNo
					= getBreaker().getBreakColumnNo(getXlsHandler(),
													rno,
													breakRowNo);
			}
			DetailDefine dd = getDetailDefines().get(ddIdx);
			List<CellDefine> cds = dd.getCellDefines();
			int cno = addValue(
							cds,
							0,
							((dd.getIterateFirstCellDefineIndex() >= 0)
							?	dd.getIterateFirstCellDefineIndex()
							:	breakColumnNo),
							rno,
							1,
							values,
							breakRowNo,
							breakColumnNo);
			if (dd.getIterateFirstCellDefineIndex() >= 0) {
				CellDefine firstCd = cds.get(
									dd.getIterateFirstCellDefineIndex());
				fillEmptyCell(cno, firstCd.getColumnNo(), values);
				cno = firstCd.getColumnNo();
				int iterateLimit = 0;
				if (breakColumnNo >= firstCd.getColumnNo()) {
					int iterateCellNum
							= cds.get(cds.size() - 1).getColumnNo()
								+ dd.getSkipCellNumAsInterating()
								- firstCd.getColumnNo() + 1;
					iterateLimit
						= (breakColumnNo
								+ dd.getSkipCellNumAsInterating()
								- firstCd.getColumnNo() + iterateCellNum)
							/ iterateCellNum;
				}
				int skipCellNum = 0;
				for (int x = 0; x < iterateLimit; ++x) {
					fillEmptyCell(0, skipCellNum, values);
					cno = addValue(
								cds,
								dd.getIterateFirstCellDefineIndex(),
								cds.size(),
								rno,
								cno + skipCellNum,
								values,
								breakRowNo,
								breakColumnNo);
					skipCellNum = dd.getSkipCellNumAsInterating();
				}
			}
			if (getDetailCallable() == null) {
				if (details != null) {
					details.add(values);
				}
			}
			else if ((rno - getFirstDetailRowNo())
								% getDetailDefines().size()
						== getDetailDefines().size() - 1) {
				detailBlock.add(values);
				if (getDetailCallable() != null) {
					getDetailCallable().call(detailBlock);
				}
				for (List<String> detailLine : detailBlock) {
					if (details != null) {
						details.add(detailLine);
					}
				}
				detailBlock = new LinkedList<List<String>>();
			}
			else {
				detailBlock.add(values);
			}
		}
	}

	/** 機能.
	 * <P>
	 * Excel ファイルの読み込みを行う {@link ReadXls}
	 * のインスタンスを生成する。
	 * </P>
	 * @param is
	 * Excel ファイルの内容を読み込む入力ストリーム InputStream
	 * @param localeInfo
	 * ロケール情報 {@link LocaleInfo}
	 * @param xlsHandler
	 * Excel ファイルの入力操作を行う依存ライブラリのハンドラ
	 * {@link XlsHandler}
	 * @param isOOXML
	 * Open Office XML 形式 (拡張子が .xlsx) の Excel ファイルであれば
	 * true を指定する。
	 * @return
	 * {@link ReadXls} のインスタンス
	 * @see XlsHandleForPoi
	 * @see XlsHandleForJxl
	 */
	public static ReadXls createXlsReader(final InputStream	is,
										final LocaleInfo	localeInfo,
										final XlsHandler	xlsHandler,
										final boolean		isOOXML) {
		ReadXls readXls = new ReadXls();
		readXls.setIs(is);
		readXls.setLocaleInfo(localeInfo);
		readXls.setXlsHandler(xlsHandler);
		readXls.setOOXML(isOOXML);
		return readXls;
	}

	/** 機能.
	 * <P>
	 * 日本国のロケール情報をもつ {@link ReadXls} インスタンスを生成する。
	 * </P>
	 * @param is
	 * Excel ファイルの内容を読み込む入力ストリーム InputStream
	 * @param xlsHandler
	 * Excel ファイルの入力操作を行う依存ライブラリのハンドラ
	 * @param isOOXML
	 * Open Office XML 形式 (拡張子が .xlsx) の Excel ファイルであれば
	 * true を指定する。
	 * @return
	 * {@link ReadXls} のインスタンス
	 * @see XlsHandleForPoi
	 * @see XlsHandleForJxl
	 */
	public static ReadXls createXlsReader(final InputStream	is,
										final XlsHandler	xlsHandler,
										final boolean		isOOXML) {
		return ReadXls.createXlsReader(is,
										new LocaleInfo(),
										xlsHandler,
										isOOXML);
	}

	/** 機能.
	 * <P>
	 * ヘッダ情報と明細情報を、Excel ファイルから読み込む。
	 * </P>
	 * <OL>
	 * <LI>
	 * {@link ReadXls#createXlsReader(InputStream, LocaleInfo, XlsHandler, boolean)}
	 * とこのメソッドの引数が正当なものであることを確認する。
	 * </LI>
	 * <LI>
	 * ヘッダか明細の入力値の出力先になるハッシュテーブル/リストが、
	 * 指定されていなければダミーを用意する。
	 * </LI>
	 * <LI>
	 * Excel ファイルを依存ライブラリのワークブックに読み込む。
	 * </LI>
	 * <LI>
	 * ワークブックからシートを読み込む。
	 * </LI>
	 * <LI>
	 * シートから読み込める最大行数を求める。
	 * </LI>
	 * <LI>
	 * ヘッダ情報を読み込む。
	 * </LI>
	 * <LI>
	 * 明細情報を読み込む。
	 * </LI>
	 * <LI>
	 * 例外 {@link ReadXlsException} が発生すれば、そのエラーコードを
	 * 戻り値に返す。
	 * </OL>
	 * @param headers
	 * ヘッダ情報から読み込んだ入力値を、項目値をキーとして、
	 * 出力するハッシュテーブル
	 * @param details
	 * 明細情報から読み込んだ入力値を、行数分、出力する
	 * String リストのリスト
	 * @param invalidCellInfos
	 * 検知した不正データの情報を出力する
	 * {@link InvalidCellInfo} のリスト
	 * @return
	 * リターンコード {@link ReturnCode}
	 * @throws IOException
	 * 入出力処理で異常が起きたら発生する。
	 */
	public final ReturnCode parse(
						final Hashtable<String, String>	headers,
						final List<List<String>>		details,
						final List<InvalidCellInfo>		invalidCellInfos)
													throws IOException {
		if (getIs() == null
		|| getLocaleInfo() == null
		|| getXlsHandler() == null
		|| !getLocaleInfo().isValid()
		|| getSheetName() == null
		|| "".equals(getSheetName())
		|| (getHeaderDefines() == null && getDetailDefines() == null)
		|| !isValidHeaderDefines()
		|| !isValidDetailDefines()
		|| getMaxInvalidCellNum() < 1
		|| (invalidCellInfos == null
			&& getInvalidCellInfoCollector() == null)) {
			return ReturnCode.ILLEGAL_ARGUMENT;
		}
		Hashtable<String, String> heads	= (headers == null)
										?	new Hashtable<String, String>()
										:	headers;
		setInvalidCellInfos(invalidCellInfos);
		ReturnCode result = ReturnCode.NORMAL;
		try {
			if (!getXlsHandler().isLoadedWorkbook()
			&& !getXlsHandler().loadWorkbook(getIs(),
												getLocaleInfo(),
												isOOXML())) {
				return ReturnCode.NOT_EXL_FILE;
			}
			if (!getXlsHandler().setSheet(getSheetName())) {
				return ReturnCode.NOT_FOUND_SHEET;
			}
			setReadableRowNum(getXlsHandler().getLastRowNum());
			if (getHeaderDefines() != null) {
				readHeaders(heads);
				if (getHeaderCallable() != null) {
					heads = getHeaderCallable().call(heads);
				}
				if (isStopWhenHeaderHasInvalidValue()
				&& getFoundInvalidCellNum() > 0) {
					result = ReturnCode.FOUND_INVALID_CELLS;
				}
			}
			if (getDetailDefines() != null) {
				readDetails(details);
			}
			if (getFoundInvalidCellNum() > 0) {
				result = ReturnCode.FOUND_INVALID_CELLS;
			}
		} catch (ReadXlsException reex) {
			result = reex.getCode();
		}
		return result;
	}

	/** 機能.
	 * <P>
	 * ワークブックとシートにかかわるリソースを解放する。
	 * <BR>
	 * Excel ファイルを読み込む入力ストリームは閉じない。
	 * </P>
	 */
	public final void close() {
		getXlsHandler().close();
	}

	/** 機能.
	 * <P>
	 * Excel ファイルからヘッダ情報と明細情報を読み込む。
	 * </P>
	 * <OL>
	 * <LI>
	 * {@link ReadXls#createXlsReader(InputStream, LocaleInfo, XlsHandler, boolean)}
	 * で ReadXls インスタンスを生成する。
	 * </LI>
	 * <LI>
	 * {@link ReadXls#parse(Hashtable, List, List)}
	 * で Excel ファイルからヘッダ情報と明細情報を読み込む。
	 * <BR>
	 * 不正データを検知すれば、その情報も出力する。
	 * </LI>
	 * <LI>
	 * {@link ReadXls#close()} で読み込み処理のためのリソースを解放する。
	 * </LI>
	 * </OL>
	 * @param is
	 * Excel ファイルの内容を読み込む入力ストリーム InputStream
	 * @param localeInfo
	 * ロケール情報 {@link LocaleInfo}
	 * @param xlsHandler
	 * Excel ファイルの入力操作を行う依存ライブラリのハンドラ
	 * @param isOOXML
	 * Open Office XML 形式 (拡張子が .xlsx) の Excel ファイルであれば
	 * true を指定する。
	 * @param sheetName
	 * シート名
	 * @param headerDefines
	 * ヘッダ情報のセル定義情報 {@link HeaderDefine}
	 * @param headerCallable
	 * 読み込んだヘッダ情報のコールバック {@link HeaderCallable}
	 * @param stopWhenHeaderHasInvalidValue
	 * ヘッダ情報の読み込みで不正データが検知されたら処理を停止する場合は
	 * true を指定する。
	 * @param firstDetailRowNo
	 * 明細情報を読み込む最初の行番号 (1～)
	 * @param breaker
	 * 読み込みを停止する行・列の番号を決定する {@link Breaker}
	 * @param detailDefines
	 * 明細情報のセル定義情報 {@link DetailDefine}
	 * @param detailCallable
	 * 明細情報が一件分（detailDefines のリスト長分の行)
	 * を読み込むたびに呼び出されるコールバック {@link DetailCallable}
	 * @param maxInvalidCellNum
	 * 不正データの情報を保持する最大個数
	 * @param headers
	 * ヘッダ情報から読み込んだ入力値を、項目値をキーとして、
	 * 出力するハッシュテーブル
	 * @param details
	 * 明細情報から読み込んだ入力値を、行数分、出力する
	 * String リストのリスト
	 * @param invalidCellInfos
	 * 検知した不正データの情報を出力する
	 * {@link InvalidCellInfo} のリスト
	 * @param invalidCellInfoCollector
	 * 不正データであるセルを検知するたびに通知する
	 * {@link InvalidCellInfoCollector}
	 * @return
	 * リターンコード {@link ReturnCode}
	 * @throws IOException
	 * 入出力処理で異常が起きたら発生する。
	 */
	public static ReturnCode readInputStream(
			final InputStream				is,
			final LocaleInfo				localeInfo,
			final XlsHandler				xlsHandler,
			final boolean					isOOXML,
			final String					sheetName,
			final List<HeaderDefine>		headerDefines,
			final HeaderCallable			headerCallable,
			final boolean					stopWhenHeaderHasInvalidValue,
			final int						firstDetailRowNo,
			final Breaker					breaker,
			final List<DetailDefine>		detailDefines,
			final DetailCallable			detailCallable,
			final int						maxInvalidCellNum,
			final Hashtable<String, String>	headers,
			final List<List<String>>		details,
			final List<InvalidCellInfo>		invalidCellInfos,
			final InvalidCellInfoCollector	invalidCellInfoCollector)
					throws IOException {
		ReadXls readXls = ReadXls.createXlsReader(is,
													localeInfo,
													xlsHandler,
													isOOXML);
		try {
			readXls.setSheetName(sheetName);
			readXls.setHeaderDefines(headerDefines);
			readXls.setHeaderCallable(headerCallable);
			readXls.setStopWhenHeaderHasInvalidValue(
											stopWhenHeaderHasInvalidValue);
			readXls.setFirstDetailRowNo(firstDetailRowNo);
			readXls.setBreaker(breaker);
			readXls.setDetailDefines(detailDefines);
			readXls.setDetailCallable(detailCallable);
			readXls.setMaxInvalidCellNum(maxInvalidCellNum);
			readXls.setInvalidCellInfoCollector(invalidCellInfoCollector);
			return readXls.parse(headers,
									details,
									invalidCellInfos);
		} finally {
			readXls.close();
		}
	}

	@Override
	public final void notifyInvalidCellInfo(
									final int				rowNo,
									final int				columnNo,
									final CellDefine		cellDefine,
									final CellTypeDefine	cellType,
									final String			messageId)
										throws ReadXlsException {
		setFoundInvalidCellNum(getFoundInvalidCellNum() + 1);
		if (getInvalidCellInfos() != null) {
			getInvalidCellInfos().add(
					new InvalidCellInfo(rowNo,
										columnNo,
										cellDefine.getName(),
										messageId,
										cellType,
										cellDefine
										));
		}
		if (getInvalidCellInfoCollector() != null) {
			getInvalidCellInfoCollector()
						.notifyInvalidCellInfo(rowNo,
												columnNo,
												cellDefine,
												cellType,
												messageId);
		}
	}

}
