package readxls;
/*-**********************************************************************
 * File: ReadXlsUtil.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;

/** Excel ファイル読み込みユーティリティ・クラス.
 * <P>
 * {@link ReadXls}
 * で読み込んだデータ内容をフォーマット編集するユーティリティ。
 * </P>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class ReadXlsUtil {

	/** 定数定義.
	 * <P>
	 * 日本の年号表記で置き換えたい箇所を
	 * SimpleDateFoemat の形式のフォーマット文字列に付加するための文字列
	 * (=""%年号%")。
	 * </P>
	 */
	public static final String JAPANESE_ERA_NAME_PLACE = "%年号%";

	/** 日本の年号を定義する内部クラス.
	 * <P>
	 * {@link ReadXlsUtil} のために日本の年号を定義する。
	 * </P>
	 *
	 * @author maoyam (maoyam@mail.goo.ne.jp)
	 */
	class JapaneseEraName {

		/** 属性.
		 * <P>
		 * 日本の年号の名称。
		 * </P>
		 */
		private String name = null;

		/** 属性.
		 * <P>
		 * 日本の年号それぞれがはじまった西暦年。
		 * </P>
		 */
		private int beginYear = Integer.MIN_VALUE;

		/** getter/setter.
		 * @return
		 * {@link JapaneseEraName#name} を返す。
		 */
		String getName() {
			return this.name;
		}

		/** getter/setter.
		 * @param name
		 * {@link JapaneseEraName#name} に指定する日本の年号の名称。
		 */
		void setName(final String name) {
			this.name = name;
		}

		/** getter/setter.
		 * @return
		 * {@link JapaneseEraName#beginYear} を返す。
		 */
		int getBeginYear() {
			return this.beginYear;
		}

		/** getter/setter.
		 * @param beginYear
		 * {@link JapaneseEraName#beginYear}
		 * に設定する日本の年号それぞれがはじまった西暦年。
		 */
		void setBeginYear(final int beginYear) {
			this.beginYear = beginYear;
		}

		/** コンストラクタ.
		 * <P>
		 * {@link ReadXlsUtil} のために日本の年号の情報を生成する。
		 * </P>
		 * @param name
		 * 日本の年号の名称。
		 * @param beginYear
		 * 日本の年号それぞれがはじまった西暦年。
		 */
		JapaneseEraName(final String name, final int beginYear) {
			this.name = name;
			this.beginYear = beginYear;
		}

	}

	/** リソース.
	 * <P>
	 * 日本の年号の情報 {@link JapaneseEraName}
	 * </P>
	 */
	private List<JapaneseEraName> japaneseEraNames = null;

	/** getter/setter.
	 * @return
	 * {@link ReadXlsUtil#japaneseEraNames} を返す。
	 */
	private List<JapaneseEraName> getJapaneseEraNames() {
		return japaneseEraNames;
	}

	/** getter/setter.
	 * @param japaneseEraNames
	 * {@link ReadXlsUtil#japaneseEraNames}
	 * に設定する
	 * {@link JapaneseEraName}
	 * のリスト
	 */
	private void setJapaneseEraNames(
							final List<JapaneseEraName> japaneseEraNames) {
		this.japaneseEraNames = japaneseEraNames;
	}

	/** コンストラクタ.
	 * <P>
	 * ユーティリティ・クラス (static メソッドだけを提供するクラス)
	 * なので、private なコンストラクタとしている。
	 * </P>
	 */
	public ReadXlsUtil() {}

	/** ユーティリティ.
	 * <P>
	 * 日本の年号の情報を初期化する。
	 * <BR>
	 * サポートしている年号は以下のとおり。
	 * </P>
	 * <UL>
	 * <LI>
	 * 慶応 (1865-1867)
	 * </LI>
	 * <LI>
	 * 明治 (1868-1911)、
	 * </LI>
	 * <LI>
	 * 大正 (1912-1925)、
	 * </LI>
	 * <LI>
	 * 昭和 (1926-1988)、
	 * </LI>
	 * <LI>
	 * 平成 (1989-)
	 * </LI>
	 * </UL>
	 */
	private void initJapaneseEraNames() {
		if (getJapaneseEraNames() == null) {
			setJapaneseEraNames(new LinkedList<JapaneseEraName>());
			getJapaneseEraNames().add(
							(new ReadXlsUtil())
								.new JapaneseEraName("慶応", 1865));
			getJapaneseEraNames().add(
							(new ReadXlsUtil())
								.new JapaneseEraName("明治", 1868));
			getJapaneseEraNames().add(
							(new ReadXlsUtil())
								.new JapaneseEraName("大正", 1912));
			getJapaneseEraNames().add(
							(new ReadXlsUtil())
								.new JapaneseEraName("昭和", 1926));
			getJapaneseEraNames().add(
							(new ReadXlsUtil())
								.new JapaneseEraName("平成", 1989));
		}
	}

	/** 機能.
	 * <P>
	 * SimpleDateFormat 形式のフォーマット文字列に
	 * {@link ReadXlsUtil#JAPANESE_ERA_NAME_PLACE}
	 * を付加した編集文字列で、日付を日本の年号を含んだ文字列に変換する。
	 * </P>
	 * @param date
	 * 変換する日付。
	 * @param pattern
	 * SimpleDateFormat 形式のフォーマット文字列に
	 * {@link ReadXlsUtil#JAPANESE_ERA_NAME_PLACE}
	 * を付加した編集文字列。
	 * @param locale
	 * SimpleDateFormat で変換するロケール {@link Locale}。
	 * @return
	 * 日本の年号を含めた日付をあらわす文字列を返す。
	 */
	public final String dateToJapaneseEraName(final Date		date,
												final String	pattern,
												final Locale	locale) {
		initJapaneseEraNames();
		GregorianCalendar cal = new GregorianCalendar();
		cal.setTime(date);
		int year = cal.get(Calendar.YEAR);
		String eraName = "西暦 G yyyy年";
		for (int i = 0; i < getJapaneseEraNames().size(); ++i) {
			JapaneseEraName jen = getJapaneseEraNames().get(i);
			if (jen.getBeginYear() <= year
			&& (i == getJapaneseEraNames().size() - 1
				|| getJapaneseEraNames().get(i + 1).getBeginYear()
					> year)) {
				int eraYear = (year - jen.getBeginYear() + 1);
				eraName = jen.getName()
						+ ((eraYear != 1)?	eraYear:	"元") + "年";
				break;
			}
		}
		return (new SimpleDateFormat(
							pattern.replaceAll(
										ReadXlsUtil.JAPANESE_ERA_NAME_PLACE,
										eraName),
							locale))
						.format(date);
	}

	/** 機能.
	 * <P>
	 * {@link ReadXlsUtil#dateToJapaneseEraName(Date, String, Locale)}
	 * を使用して、
	 * 西暦での日付をあらわす文字列から日本の年号を含む文字列に変換する。
	 * </P>
	 * @param date
	 * 西暦での日付をあらわす文字列。
	 * @param datePattern
	 * 西暦での日付をあらわす文字列を Date に型変換するための
	 * SimpleDateFormat 形式の変換文字列。
	 * @param pattern
	 * SimpleDateFormat 形式のフォーマット文字列に
	 * {@link ReadXlsUtil#JAPANESE_ERA_NAME_PLACE}
	 * を付加した編集文字列。
	 * @param locale
	 * SimpleDateFormat で変換するロケール {@link Locale}。
	 * @return
	 * 日本の年号を含めた日付をあらわす文字列を返す。
	 * @throws ParseException
	 * 西暦での日付をあらわす文字列を
	 * Date に変換する際に不正な変換文字列であった場合に発生する。
	 */
	public final String dateToJapaneseEraName(final String		date,
												final String	datePattern,
												final String	pattern,
												final Locale	locale)
													throws ParseException {
		return dateToJapaneseEraName(
							(new SimpleDateFormat(datePattern, locale))
									.parse(date),
							pattern,
							locale);
	}

	/** ユーティリティ.
	 * <P>
	 * ロケールに対応する数値変換フォーマッタを
	 * {@link NumberFormat#getNumberInstance(Locale)}
	 * で生成する。
	 * </P>
	 * @param locale
	 * ロケール {@link Locale}
	 * @return
	 * 数値変換フォーマッタ {@link NumberFormat}
	 */
	private static NumberFormat makeNf(final Locale locale) {
		return NumberFormat.getNumberInstance(locale);
	}

	/** 機能.
	 * <P>
	 * 実数 double をロケールに対応した数値に変換する。
	 * </P>
	 * @param number
	 * 実数 double
	 * @param locale
	 * ロケール {@link Locale}
	 * @return
	 * 変換結果の文字列を返す。
	 */
	public static String numberFormat(final double		number,
										final Locale	locale) {
		return ReadXlsUtil.makeNf(locale).format(number);
	}

	/** 機能.
	 * <P>
	 * 整数 long をロケールに対応した数値に変換する。
	 * </P>
	 * @param number
	 * 整数 long
	 * @param locale
	 * ロケール {@link Locale}
	 * @return
	 * 変換結果の文字列を返す。
	 */
	public static String numberFormat(final long		number,
										final Locale	locale) {
		return ReadXlsUtil.makeNf(locale).format(number);
	}

	/** 機能.
	 * <P>
	 * 任意精度の数をロケールに対応した数値に変換する。
	 * </P>
	 * @param number
	 * 任意精度の数 {@link BigDecimal}
	 * @param locale
	 * ロケール {@link Locale}
	 * @return
	 * 変換結果の文字列を返す。
	 */
	public static String numberFormat(final BigDecimal	number,
										final Locale	locale) {
		return	(number != null)
				?	(number.scale() > 0)
					?	ReadXlsUtil.numberFormat(number.doubleValue(),
													locale)
					:	ReadXlsUtil.numberFormat(number.longValue(),
													locale)
				:	ReadXls.EMPTY_VALUE;
	}

	/** 機能.
	 * <P>
	 * 数を表す文字列をロケールに対応した数値に変換する。
	 * </P>
	 * @param number
	 * 数を表す文字列
	 * @param locale
	 * ロケール {@link Locale}
	 * @return
	 * 変換結果の文字列を返す。
	 */
	public static String numberFormat(final String		number,
										final Locale	locale) {
		return	(number != null)
				?	ReadXlsUtil.numberFormat(new BigDecimal(number),
											locale)
				:	ReadXls.EMPTY_VALUE;
	}

}
