package readxls.control;
/*-**********************************************************************
 * File: DetailDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.util.List;

import readxls.XlsHandler;

/** 明細情報構成.
 * <P>
 * 明細情報を構成する行とセルの構成を定義する。
 * </P>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class DetailDefine {

	/** 定数.
	 * <P>
	 * 列方向に繰り返し読み込むセルがない場合の
	 * {@link DetailDefine#cellDefines}
	 * の添え字 (= -1)
	 * </P>
	 */
	public static final int NO_ITERATE_CELL = -1;

	/** 明細情報定義.
	 * <P>
	 * 列方向に繰り返し読み込むセルの定義リスト
	 * {@link DetailDefine#cellDefines}
	 * の添え字。
	 * <BR>
	 * 繰り返し読み込むセル定義がないならば、
	 * {@link DetailDefine#NO_ITERATE_CELL}
	 * を指定する。
	 * </P>
	 */
	private int iterateFirstCellDefineIndex = DetailDefine.NO_ITERATE_CELL;

	/** 明細情報定義.
	 * <P>
	 * セルを列方向に繰り返し読み込む際にフィラーのセルをいくつ読み飛ばすのかを指定する。
	 * </P>
	 */
	private int skipCellNumAsInterating = 0;

	/** 明細情報定義.
	 * <P>
	 * 明細情報を構成するセルの定義を指定する。
	 * </P>
	 * @see CellDefine
	 */
	private List<CellDefine> cellDefines = null;

	/** getter/setter.
	 * @return
	 * {@link DetailDefine#iterateFirstCellDefineIndex}
	 * を返す。
	 */
	public final int getIterateFirstCellDefineIndex() {
		return this.iterateFirstCellDefineIndex;
	}

	/** getter/setter.
	 * @param iterateFirstCellDefineIndex
	 * {@link DetailDefine#iterateFirstCellDefineIndex}
	 * に設定する値。
	 */
	public final void setIterateFirstCellDefineIndex(
									final int iterateFirstCellDefineIndex) {
		this.iterateFirstCellDefineIndex = iterateFirstCellDefineIndex;
	}

	/** getter/setter.
	 * @return
	 * {@link DetailDefine#skipCellNumAsInterating}
	 * を返す。
	 */
	public final int getSkipCellNumAsInterating() {
		return this.skipCellNumAsInterating;
	}

	/** getter/setter.
	 * @param skipCellNumAsInterating
	 * {@link DetailDefine#skipCellNumAsInterating}
	 * に設定する値 (≧1)。
	 */
	public final void setSkipCellNumAsInterating(
									final int skipCellNumAsInterating) {
		this.skipCellNumAsInterating = skipCellNumAsInterating;
	}

	/** getter/setter.
	 * @return
	 * セル定義のリスト
	 * {@link DetailDefine#cellDefines}
	 * を返す。
	 * @see CellDefine
	 */
	public final List<CellDefine> getCellDefines() {
		return this.cellDefines;
	}

	/** getter/setter.
	 * @param cellDefines
	 * {@link DetailDefine#cellDefines}
	 * に設定するセル定義のリスト。
	 * @see CellDefine
	 */
	public final void setCellDefines(final List<CellDefine> cellDefines) {
		this.cellDefines = cellDefines;
	}

	/** コンストラクタ.
	 * @param iterateFirstCellDefineIndex
	 * {@link DetailDefine#iterateFirstCellDefineIndex}
	 * に設定する値。
	 * @param skipCellNumAsInterating
	 * {@link DetailDefine#skipCellNumAsInterating}
	 * に設定する値。
	 * @param cellDefines
	 * {@link DetailDefine#cellDefines}
	 * に設定するセル定義のリスト。
	 * @see CellDefine
	 */
	public DetailDefine(
					final int				iterateFirstCellDefineIndex,
					final int				skipCellNumAsInterating,
					final List<CellDefine>	cellDefines) {
		this.iterateFirstCellDefineIndex	= iterateFirstCellDefineIndex;
		this.skipCellNumAsInterating		= skipCellNumAsInterating;
		this.cellDefines					= cellDefines;
	}

	/** 機能.
	 * <P>
	 * 明細情報の定義内容が正当なものであることを確認する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール情報
	 * @return
	 * 繰り返し読み込む前に読み飛ばすセル数
	 * {@link DetailDefine#skipCellNumAsInterating}
	 * がゼロ以上で、
	 * セル定義のリスト
	 * {@link DetailDefine#cellDefines}
	 * が null ではなく、
	 * {@link DetailDefine#iterateFirstCellDefineIndex}
	 * が -1 からセル定義リストの個数未満の値であれば true
	 * を、そうでなｋれば false を返す。
	 */
	public final boolean isValid(final XlsHandler	xlsHandler,
								final LocaleInfo	localeInfo) {
		boolean result = (getSkipCellNumAsInterating() >= 0
							&& getCellDefines() != null
							&& getIterateFirstCellDefineIndex()
									>= DetailDefine.NO_ITERATE_CELL
							&& getIterateFirstCellDefineIndex()
									< getCellDefines().size());
		if (result) {
			for (CellDefine cd : getCellDefines()) {
				result = cd.isValidParameters(xlsHandler, localeInfo);
				if (!result) {
					break;
				}
			}
		}
		return result;
	}

}
