package readxls.control;
/*-**********************************************************************
 * File: RowColumnBreaker.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.util.List;

import readxls.XlsHandler;

/** 行・列の読み込み終了位置制御.
 * <P>
 * シートから明細情報を読み込む際の、行と列の読み込み終了位置を求める。
 * </P>
 * @see Breaker
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class RowColumnBreaker implements Breaker {

	/** 制御パラメータ.
	 * <P>
	 * 必須であるセルの列番号 (1～) のリスト。
	 * <BR>
	 * この制御パラメータが null であれば、
	 * シートに入力されている最後の行まで読み込む。
	 * <BR>
	 * この制御パラメータが空リストであれば、
	 * 行のすべてのセルが未入力である行の連続を
	 * 読み込み停止位置を求めるためにシートから探す。
	 * </P>
	 */
	private List<Integer> mandatoryColumnNos = null;

	/** 制御パラメータ.
	 * <P>
	 * 読み込みを停止する行のあとに、
	 * 必須であるセルが入力されていない行が連続する行数 (≧1)。
	 * </P>
	 */
	private int maxBlankRowNum = 0;

	/** 制御パラメータ.
	 * <P>
	 * 読み込みを停止する列の列番号 (1～)。
	 * <BR>
	 * ゼロ以下の値を設定した場合、この制御パラメータは無視される。
	 * </P>
	 */
	private int fixedColumnNo = -1;

	/** 制御パラメータ.
	 * <P>
	 * 明細情報を構成する行のどの行の入力状態で読み込み停止位置を
	 * 求めるかを指定する。
	 * <BR>
	 * 明細情報を構成する最初の行はゼロで指定する。
	 * <BR>
	 * ゼロ未満の値を設定した場合、この制御パラメータは無視される。
	 * </P>
	 *
	 */
	private int offsetOfReadRowIndex = -1;

	/** 制御パラメータ.
	 * <P>
	 * 列の読み込み停止位置の右隣にいくつのセルが連続するかを指定する
	 * (≧1)。
	 * </P>
	 */
	private int maxBlankColumnNum = 0;

	/** getter/setter.
	 * @return
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * を返す。
	 */
	public final List<Integer> getMandatoryColumnNos() {
		return this.mandatoryColumnNos;
	}

	/** getter/setter.
	 * @param mandatoryColumnNos
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * に設定する必須入力のセルの列番号 (1～) のリスト。
	 */
	public final void setMandatoryColumnNos(
								final List<Integer> mandatoryColumnNos) {
		this.mandatoryColumnNos = mandatoryColumnNos;
	}

	/** getter/setter.
	 * @return
	 * {@link RowColumnBreaker#maxBlankRowNum}
	 * を返す。
	 */
	public final int getMaxBlankRowNum() {
		return this.maxBlankRowNum;
	}

	/** getter/setter.
	 * @param maxBlankRowNum
	 * {@link RowColumnBreaker#maxBlankRowNum}
	 * に設定する値。
	 */
	public final void setMaxBlankRowNum(final int maxBlankRowNum) {
		this.maxBlankRowNum = maxBlankRowNum;
	}

	/** getter/setter.
	 * @return
	 * {@link RowColumnBreaker#fixedColumnNo}
	 * を返す。
	 */
	public final int getFixedColumnNo() {
		return this.fixedColumnNo;
	}

	/** getter/setter.
	 * @param fixedColumnNo
	 * {@link RowColumnBreaker#fixedColumnNo}
	 * に設定する値。
	 */
	public final void setFixedColumnNo(final int fixedColumnNo) {
		this.fixedColumnNo = fixedColumnNo;
	}

	/** getter/setter.
	 * @return
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * を返す。
	 */
	public final int getOffsetOfReadRowIndex() {
		return this.offsetOfReadRowIndex;
	}

	/** getter/setter.
	 * @param offsetOfReadRowIndex
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * に設定する値。
	 */
	public final void setOffsetOfReadRowIndex(
										final int offsetOfReadRowIndex) {
		this.offsetOfReadRowIndex = offsetOfReadRowIndex;
	}

	/** getter/setter.
	 * @return
	 * {@link RowColumnBreaker#maxBlankColumnNum}
	 * を返す。
	 */
	public final int getMaxBlankColumnNum() {
		return this.maxBlankColumnNum;
	}

	/** getter/setter.
	 * @param maxBlankColumnNum
	 * {@link RowColumnBreaker#maxBlankColumnNum}
	 * に設定する値。
	 */
	public final void setMaxBlankColumnNum(final int maxBlankColumnNum) {
		this.maxBlankColumnNum = maxBlankColumnNum;
	}

	/** コンストラクタ.
	 * <P>
	 * 行・列の読み込み停止位置を制御する {@link Breaker}
	 * のコンストラクタ。
	 * </P>
	 * @param mandatoryColumnNos
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * の設定値。
	 * @param maxBlankRowNum
	 * {@link RowColumnBreaker#maxBlankRowNum}
	 * の設定値。
	 * @param fixedColumnNo
	 * {@link RowColumnBreaker#fixedColumnNo}
	 * の設定値。
	 * @param offsetOfReadRowIndex
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * の設定値。
	 * @param maxBlankColumnNum
	 * {@link RowColumnBreaker#maxBlankColumnNum}
	 * の設定値。
	 */
	public RowColumnBreaker(final List<Integer>	mandatoryColumnNos,
							final int			maxBlankRowNum,
							final int			fixedColumnNo,
							final int			offsetOfReadRowIndex,
							final int			maxBlankColumnNum) {
		this.mandatoryColumnNos		= mandatoryColumnNos;
		this.maxBlankRowNum			= maxBlankRowNum;
		this.fixedColumnNo			= fixedColumnNo;
		this.offsetOfReadRowIndex	= offsetOfReadRowIndex;
		this.maxBlankColumnNum		= maxBlankColumnNum;
	}

	/** 機能.
	 * <P>
	 * 行・列の読み込みを停止する位置を求める制御パラメータが
	 * 正当であることを確認する。
	 * </P>
	 * @return
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * が null で
	 * {@link RowColumnBreaker#maxBlankRowNum}
	 * がゼロ以下、かつ、
	 * {@link RowColumnBreaker#fixedColumnNo}
	 * がゼロ以下で
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * がゼロ以上で、
	 * {@link RowColumnBreaker#maxBlankColumnNum}
	 * がゼロ以下の場合は false
	 * を、そうでなければ true を返す。
	 */
	@Override
	public final boolean isValid() {
		return (getMandatoryColumnNos() == null
					|| getMaxBlankRowNum() > 0)
				&& (getFixedColumnNo() > 0
					|| getOffsetOfReadRowIndex() < 0
					|| getMaxBlankColumnNum() > 0);
	}

	/** ユーティリティ.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装。
	 * @param mandatoryColumnsNum
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * の要素数。
	 * @return
	 * ある行のすべての必須入力セルが未入力であれば true
	 * を、そうでなければ false を返す。
	 */
	private boolean areAllColumnsEmpty(
								final XlsHandler	xlsHandler,
								final int			mandatoryColumnsNum) {
		boolean result = true;
		for (int i = 0; i < mandatoryColumnsNum; ++i) {
			int columnNo = getMandatoryColumnNos().get(i);
			xlsHandler.readInBreakerCell(columnNo);
			if (!xlsHandler.isBreakerCellEmpty()) {
				result = false;
				break;
			}
		}
		return result;
	}

	/** 機能.
	 * <P>
	 * 明細情報の読み込みを停止する行の行番号を求める。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param firstDetailRowNo
	 * 明細情報を構成する最初の行に当たるシートの読み込み行の番号 (1～)
	 * @param readableRowNum
	 * シートにデータ入力されているセルがある最後の行の番号 (1～)
	 * @return
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * が null ならば、シートの最後の行の行番号 (1～) を返す。
	 * <BR>
	 * あるいは、
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * が空リストで
	 * {@link RowColumnBreaker#maxBlankRowNum}
	 * に指定されている行数の空行が連続している場合、
	 * その直前の行の行番号 (1～) を返す。
	 * <BR>
	 * あるいは、
	 * {@link RowColumnBreaker#mandatoryColumnNos}
	 * が空リストでなく、
	 * {@link RowColumnBreaker#maxBlankRowNum}
	 * に指定されている行数の必須入力セルが未入力である行が
	 * 連続している場合、その連続の直前の行の行番号 (1～) を返す。
	 */
	@Override
	public final int getBreakRowNo(final XlsHandler	xlsHandler,
									final int		firstDetailRowNo,
									final int		readableRowNum) {
		if (getMandatoryColumnNos() == null) {
			return readableRowNum;
		}
		int mandatoryColumnsNum	= getMandatoryColumnNos().size();
		int breakRowNo			= firstDetailRowNo - 1;
		int blankRowNum			= 0;
		for (int r = firstDetailRowNo; r <= readableRowNum; ++r) {
			xlsHandler.readInBreakerCells(r);
			if ((mandatoryColumnsNum == 0
				&& xlsHandler.getBreakerCellsLength() == 0)
			|| (mandatoryColumnsNum > 0
					&& !areAllColumnsEmpty(xlsHandler,
											mandatoryColumnsNum))) {
				if (++blankRowNum >= getMaxBlankRowNum()) {
					break;
				}
			}
			else {
				breakRowNo	= r;
				blankRowNum	= 0;
			}
		}
		return breakRowNo;
	}

	/** 機能.
	 * <P>
	 * 明細情報を読み込身を停止する列の列番号を求める。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param rno
	 * 読み込む行の行番号 (1～)
	 * @param breakRowNo
	 * 読み込みを終了する行の行番号 (1～)
	 * @return
	 * {@link RowColumnBreaker#fixedColumnNo}
	 * がゼロ以下、
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * がゼロ未満で
	 * {@link RowColumnBreaker#maxBlankColumnNum}
	 * がゼロ以下の場合で、
	 * 引数 rno が読み込みを停止する行番号を超えていればゼロを、
	 * そうでなければ明細情報を構成する最初の行に入力されている最右のセルの
	 * 列番号を返す。
	 * <BR>
	 * あるいは、
	 * {@link RowColumnBreaker#fixedColumnNo}
	 * が 1 以上の場合は、その設定値を返す。
	 * <BR>
	 * あるいは、
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * がゼロ未満であるか、または、
	 * 引数 rno に
	 * {@link RowColumnBreaker#offsetOfReadRowIndex}
	 * を加えた読み込み停止列番号を求める対象行の魚番号が、
	 * 引数 breakRowNo を超えている場合は、ゼロを返す。
	 * <BR>
	 * あるいは、
	 * {@link RowColumnBreaker#maxBlankColumnNum}
	 * に指定された列数分の空セルが連続している箇所の左隣の列の列番号を
	 * 返す。
	 */
	@Override
	public final int getBreakColumnNo(final XlsHandler	xlsHandler,
										final int		rno,
										final int		breakRowNo) {
		if (getFixedColumnNo() < 1
		&& getOffsetOfReadRowIndex() < 0
		&& getMaxBlankColumnNum() < 1) {
			if (breakRowNo < rno) {
				return 0;
			}
			xlsHandler.readInBreakerCells(rno);
			return xlsHandler.getBreakerCellsLength();
		}
		if (getFixedColumnNo() > 0) {
			return getFixedColumnNo();
		}
		int rowNo = rno + getOffsetOfReadRowIndex();
		if (getOffsetOfReadRowIndex() < 0 || breakRowNo < rowNo) {
			return 0;
		}
		xlsHandler.readInBreakerCells(rowNo);
		int breakColumnNo	= 0;
		int blankColumnCnt	= 0;
		int limit = xlsHandler.getBreakerCellsLength();
		for (int i = 0; i < limit; ++i) {
			xlsHandler.readInBreakerCell(i + 1);
			if (xlsHandler.isBreakerCellEmpty()) {
				if (++blankColumnCnt >= getMaxBlankColumnNum()) {
					break;
				}
			}
			else {
				breakColumnNo	= i + 1;
				blankColumnCnt	= 0;
			}
		}
		return breakColumnNo;
	}

}
