package readxls.type;
/*-**********************************************************************
 * File: BooleanCellDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import readxls.XlsHandler;
import readxls.control.CellDefine;
import readxls.control.LocaleInfo;

/** 読み込みセルの項目定義.
 * <P>
 * 論理型のセルを読み込むための項目定義を行う。
 * <BR>
 * 指定する定義情報は以下のとおり。
 * </P>
 * <UL>
 * <LI>
 * セルを読み込む列の番号 (1 ～)
 * </LI>
 * <LI>
 * 項目名
 * </LI>
 * <LI>
 * 入力必須であるときに未入力だった場合のメッセージ
 * </LI>
 * <LI>
 * セルの型が論理値でなかった場合のメッセージ
 * </LI>
 * <LI>
 * セルの入力値が真または偽であるときの出力値
 * </LI>
 * </UL>
 * @see CellDefine
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class BooleanCellDefine extends CellDefine {

	/** 定数定義.
	 * <BR>
	 * セルから読み込んだ論理値が真であるときの出力値の既定値 ("true")。
	 */
	public static final String VALUE_OF_TRUE = Boolean.TRUE.toString();

	/** 定数定義.
	 * <BR>
	 * セルから読み込んだ論理値が偽であるときの出力値の既定値 ("false")。
	 */
	public static final String VALUE_OF_FALSE = Boolean.FALSE.toString();

	/** 項目定義情報.
	 * <BR>
	 * セルから読み込んだ論理値が真であるときの出力値。
	 * @see BooleanCellDefine#VALUE_OF_TRUE
	 */
	private String valueOfTrue = BooleanCellDefine.VALUE_OF_TRUE;

	/** 項目定義情報.
	 * <BR>
	 * セルから読み込んだ論理値が偽であるときの出力値。
	 * @see BooleanCellDefine#VALUE_OF_FALSE
	 */
	private String valueOfFalse = BooleanCellDefine.VALUE_OF_FALSE;

	/** getter/setter.
	 * <BR>
	 * {@link BooleanCellDefine#valueOfTrue}
	 * の getter 。
	 * @return BooleanCellDefine#valueOfTrue の値
	 */
	public final String getValueOfTrue() {
		return this.valueOfTrue;
	}

	/** getter/setter.
	 * <BR>
	 * {@link BooleanCellDefine#valueOfTrue}
	 * の setter 。
	 * @param valueOfTrue BooleanCellDefine#valueOfTrue に設定する値
	 */
	public final void setValueOfTrue(final String valueOfTrue) {
		this.valueOfTrue = valueOfTrue;
	}

	/** getter/setter.
	 * <BR>
	 * {@link BooleanCellDefine#valueOfFalse}
	 * の getter 。
	 * @return BooleanCellDefine#valueOfFalse の値
	 */
	public final String getValueOfFalse() {
		return this.valueOfFalse;
	}

	/** getter/setter.
	 * <BR>
	 * {@link BooleanCellDefine#valueOfFalse}
	 * の setter 。
	 * @param valueOfFalse BooleanCellDefine#valueOfFalse に設定する値
	 */
	public final void setValueOfFalse(final String valueOfFalse) {
		this.valueOfFalse = valueOfFalse;
	}

	/** コンストラクタ.
	 * <P>
	 * {@link BooleanCellDefine#valueOfTrue}
	 * と
	 * {@link BooleanCellDefine#valueOfFalse}
	 * の値も指定できる。
	 * </P>
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param name
	 * 項目名
	 * @param lostMandatoryMessageID
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * @param invalidTypeMessageId
	 * 列番号のセルが論理値型でなかったことを示すメッセージ
	 * @param valueOfTrue
	 * セルの入力値が真であったときの出力値
	 * @param valueOfFalse
	 * セルの入力値が偽であったときの出力値
	 */
	public BooleanCellDefine(final int			columnNo,
							final String		name,
							final String		lostMandatoryMessageID,
							final String		invalidTypeMessageId,
							final String		valueOfTrue,
							final String		valueOfFalse) {
		super(columnNo,
				name,
				lostMandatoryMessageID,
				invalidTypeMessageId,
				null);
		this.valueOfTrue	= valueOfTrue;
		this.valueOfFalse	=  valueOfFalse;
	}

	/** コンストラクタ.
	 * <P>
	 * {@link BooleanCellDefine#valueOfTrue}
	 * と
	 * {@link BooleanCellDefine#valueOfFalse}
	 * の値は既定値になる。
	 * </P>
	 * @param columnNo 読み込むセルの列番号 (1 ～)
	 * @param name 項目名
	 * @param lostMandatoryMessageID
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * @param invalidTypeMessageId
	 * 列番号のセルが論理値型でなかったことを示すメッセージ
	 */
	public BooleanCellDefine(final int		columnNo,
							final String	name,
							final String	lostMandatoryMessageID,
							final String	invalidTypeMessageId) {
		this(columnNo,
				name,
				lostMandatoryMessageID,
				invalidTypeMessageId,
				BooleanCellDefine.VALUE_OF_TRUE,
				BooleanCellDefine.VALUE_OF_FALSE);
	}

	/** 機能.
	 * <P>
	 * 論理型セルの入力値が真の場合と偽の場合のそれぞれの文字列変換結果が
	 * 設定されていることを確認する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * セルの入力値が真である場合あるいは偽である場合に対応する文字列が
	 * 定義されていなければ false
	 * を、そうでなければ true を返す。
	 */
	@Override
	protected final boolean checkParameters(
										final XlsHandler	xlsHandler,
										final LocaleInfo	localeInfo) {
		return getValueOfTrue() != null
				&& getValueOfFalse() != null
				&& getChecker() == null;
	}

	/** 機能.
	 * <P>
	 * 読み込むセルの型が論理型
	 * {@link CellTypeDefine#BOOLEAN}
	 * であることを確認する。
	 * </P>
	 * @param cellType
	 * セルの型の種別 {@link CellTypeDefine}
	 * @return
	 * 読み込むセルの型が論理値型であれば true
	 * を、そうでなければ false を返す。
	 */
	@Override
	protected final boolean checkCellType(final CellTypeDefine cellType) {
		return	CellTypeDefine.BOOLEAN == cellType;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * 確認結果が不正であった場合それぞれのメッセージが出力される。
	 * @return
	 * 常に true を返す。
	 */
	@Override
	protected final boolean isValidCellValue(final XlsHandler	xlsHandler,
											final StringBuilder msgIdSb) {
		return true;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * {@link XlsHandler#getBooleanCellValue()}
	 * でセルから入力値を読み込み、それが真であれば
	 * {@link BooleanCellDefine#getValueOfTrue()}
	 * 、そうでなければ
	 * {@link BooleanCellDefine#getValueOfFalse()}
	 * で取得した文字列を返す。
	 */
	@Override
	protected final String convertCellValue(final XlsHandler xlsHandler) {
		return	(xlsHandler.getBooleanCellValue())
				?	getValueOfTrue()
				:	getValueOfFalse();
	}

}
