package readxls.type;
/*-**********************************************************************
 * File: EmptyCellDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import readxls.ReadXls;
import readxls.XlsHandler;
import readxls.control.CellDefine;
import readxls.control.LocaleInfo;

/** 読み込みセルの項目定義.
 * <P>
 * フィラーを設定するために空白セルの項目定義を行う。
 * <BR>
 * 指定する定義情報は以下のとおり。
 * </P>
 * <UL>
 * <LI>
 * セルを読み込む列の番号 (1 ～)
 * </LI>
 * <LI>
 * 項目名
 * </LI>
 * <LI>
 * 入力必須にはできない。
 * </LI>
 * <LI>
 * セルの型が項目定義と異なる場合のメッセージは
 * {@link EmptyCellDefine#EMPTY_CELL_INVALID_TYPE_MESSAGE}
 * </LI>
 * </UL>
 * @see CellDefine
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class EmptyCellDefine extends CellDefine {

	/** 定数定義.
	 * <P>
	 * 空白セルが必須項目とされていた
	 * 実際にはありえない）場合のメッセージ
	 * </P>
	 */
	public static final String EMPTY_CELL_INVALID_TYPE_MESSAGE
														= "(empty cell)";

	/** コンストラクタ.
	 * <P>
	 * 空白セルをフィラーとして指定する場合に使用する。
	 * </P>
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param name
	 * 項目名
	 */
	public EmptyCellDefine(final int columnNo, final String name) {
		super(columnNo,
			name,
			null,
			EmptyCellDefine.EMPTY_CELL_INVALID_TYPE_MESSAGE,
			null);
	}

	/** コンストラクタ.
	 * <P>
	 * 空白セルをフィラーとして指定する場合に使用する。
	 * </P>
	 * <P>
	 * 項目名は
	 * {@link ReadXls#EMPTY_VALUE}
	 * に設定される。
	 * </P>
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 */
	public EmptyCellDefine(final int columnNo) {
		this(columnNo, ReadXls.EMPTY_VALUE);
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * 常に true を返す。
	 */
	@Override
	protected final boolean checkParameters(
										final XlsHandler	xlsHandler,
										final LocaleInfo	localeInfo) {
		return true;
	}

	/** 機能.
	 * @param cellType
	 * セルの型の種別 {@link CellTypeDefine}
	 * @return
	 * 常に true を返す。
	 */
	@Override
	protected final boolean checkCellType(final CellTypeDefine cellType) {
		return true;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * 確認結果が不正であった場合それぞれのメッセージが出力される。
	 * @return
	 * 常に true を返す。
	 */
	@Override
	protected final boolean isValidCellValue(final XlsHandler	xlsHandler,
											final StringBuilder	msgIdSb) {
		return true;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * 常に
	 * {@link ReadXls#EMPTY_VALUE}
	 * を返す。
	 */
	@Override
	protected final String convertCellValue(final XlsHandler xlsHandler) {
		return ReadXls.EMPTY_VALUE;
	}

}
