package readxls.type;
/*-**********************************************************************
 * File: NumberCellDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.math.BigDecimal;
import java.math.RoundingMode;

import readxls.XlsHandler;
import readxls.control.CellDefine;
import readxls.control.LocaleInfo;
import readxls.validate.Checker;
import readxls.validate.RangeChecker;


/** 読み込みセルの項目定義.
 * <P>
 * 数値型のセルを読み込むための項目定義を行う。
 * <BR>
 * 指定する定義情報は以下のとおり。
 * </P>
 * <UL>
 * <LI>
 * セルを読み込む列の番号 (1 ～)
 * </LI>
 * <LI>
 * 項目名
 * </LI>
 * <LI>
 * 入力必須であるときに未入力だった場合のメッセージ
 * </LI>
 * <LI>
 * セルの型が項目定義と異なる場合のメッセージ
 * </LI>
 * <LI>
 * 桁数 (整数部の桁数と小数部の桁数の和)
 * </LI>
 * <LI>
 * 小数部の桁数
 * </LI>
 * <LI>
 * セルの入力値が値域からはずれていた場合のメッセージ。
 * <BR>
 * 値域は、-1e(整数部の桁数) &lt; (セルの入力値) &lt; 1e(整数部の桁数)
 * </LI>
 * </UL>
 * @see CellDefine
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class NumberCellDefine extends CellDefine {

	/** 項目定義情報.
	 * <P>
	 * 数値の整数部と小数部の最大桁数の和
	 * </P>
	 */
	private int allDigits = 0;

	/** 項目定義情報.
	 * <P>
	 * 小数部の桁数。
	 * <BR>
	 * この桁の次の桁でセルの入力値は丸められる。
	 * </P>
	 */
	private int fractionDigits = 0;

	/** 項目定義情報.
	 * <P>
	 * 丸めのモード {@link RoundingMode}
	 * </P>
	 */
	private RoundingMode roundingMode = RoundingMode.HALF_UP;

	/** 項目定義情報.
	 * <P>
	 * {@link NumberCellDefine#allDigits}
	 * と
	 * {@link NumberCellDefine#fractionDigits}
	 * で指定される数値の値域からはずれた入力値であることを示すメッセージ
	 * </P>
	 */
	private String outOfRangeMessageId = null;

	/** フィールド.
	 * <P>
	 * セルの入力値を
	 * {@link NumberCellDefine#fractionDigits}
	 * の次の小数位桁で
	 * {@link NumberCellDefine#roundingMode}
	 * モードの丸めをした結果の数値。
	 * </P>
	 */
	private BigDecimal roundedValue = null;

	/** getter/setter.
	 * @return
	 * {@link NumberCellDefine#allDigits}
	 * の値
	 */
	public final int getAllDigits() {
		return this.allDigits;
	}

	/** getter/setter.
	 * @param allDigits
	 * {@link NumberCellDefine#allDigits}
	 * に設定する値
	 */
	public final void setAllDigits(final int allDigits) {
		this.allDigits = allDigits;
	}

	/** getter/setter.
	 * @return
	 * {@link NumberCellDefine#fractionDigits}
	 * の値
	 */
	public final int getFractionDigits() {
		return this.fractionDigits;
	}

	/** getter/setter.
	 * @param fractionDigits
	 * {@link NumberCellDefine#fractionDigits}
	 * に設定する値
	 */
	public final void setFractionDigits(final int fractionDigits) {
		this.fractionDigits = fractionDigits;
	}

	/** getter/setter.
	 * @return
	 * {@link NumberCellDefine#roundingMode}
	 * の値
	 */
	public final RoundingMode getRoundingMode() {
		return this.roundingMode;
	}

	/** getter/setter.
	 * @param roundingMode
	 * {@link NumberCellDefine#roundingMode}
	 * に設定する値
	 */
	public final void setRoundingMode(final RoundingMode roundingMode) {
		this.roundingMode = roundingMode;
	}

	/** getter/setter.
	 * @return
	 * {@link NumberCellDefine#outOfRangeMessageId}
	 * の値
	 */
	public final String getOutOfRangeMessageId() {
		return this.outOfRangeMessageId;
	}

	/** getter/setter.
	 * @param outOfRangeMessageId
	 * {@link NumberCellDefine#outOfRangeMessageId}
	 * に設定する値
	 */
	public final void setOutOfRangeMessageId(
									final String outOfRangeMessageId) {
		this.outOfRangeMessageId = outOfRangeMessageId;
	}

	/** getter/setter.
	 * @return
	 * {@link NumberCellDefine#roundedValue}
	 * の値
	 */
	public final BigDecimal getRoundedValue() {
		return this.roundedValue;
	}

	/** getter/setter.
	 * @param roundedValue
	 * {@link NumberCellDefine#roundedValue}
	 * に設定する値
	 */
	private void setRoundedValue(final BigDecimal roundedValue) {
		this.roundedValue = roundedValue;
	}

	/** コンストラクタ.
	 * <P>
	 * {@link NumberCellDefine#allDigits}
	 * 、
	 * {@link NumberCellDefine#fractionDigits}
	 * 、
	 * {@link NumberCellDefine#roundingMode}
	 * そして
	 * {@link NumberCellDefine#outOfRangeMessageId}
	 * の値も指定できる。
	 * </P>
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param name
	 * 項目名
	 * @param lostMandatoryMessageID
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * @param invalidTypeMessageId
	 * 列番号のセルが数値型でなかったことを示すメッセージ
	 * @param checker
	 * 数値型セルの入力値を BigDecimal でとり、その正当性を確認する手順
	 * {@link RangeChecker}
	 * @param allDigits
	 * 数値の整数部と小数部の最大桁数の和
	 * @param fractionDigits
	 * 小数部の桁数
	 * @param roundingMode
	 * 丸めのモード {@link RoundingMode}
	 * @param outOfRangeMessageId
	 * 整数部の桁数 (= allDigits - fractionDigits)
	 * で定義される数値の定義域
	 * -1e(整数部桁数)≦入力値≦1e(整数部桁数)
	 * の範囲外の数値が入力されていたことを示すメッセージ
	 */
	public NumberCellDefine(final int			columnNo,
							final String		name,
							final String		lostMandatoryMessageID,
							final String		invalidTypeMessageId,
							final Checker		checker,
							final int			allDigits,
							final int			fractionDigits,
							final RoundingMode	roundingMode,
							final String		outOfRangeMessageId) {
		super(columnNo,
				name,
				lostMandatoryMessageID,
				invalidTypeMessageId,
				checker);
		this.allDigits				= allDigits;
		this.fractionDigits			= fractionDigits;
		this.roundingMode			= roundingMode;
		this.outOfRangeMessageId	= outOfRangeMessageId;
	}

	/** 機能.
	 * <P>
	 * 数値型セルの総桁数と小数部桁数がゼロ以上の値を設定されていることと、
	 * 総桁数≧小数部桁数であることを確認する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * 総桁数と小数部桁数が条件を満たしていれば true
	 * を、そうでなければ false を返す。
	 */
	@Override
	protected final boolean checkParameters(
										final XlsHandler	xlsHandler,
										final LocaleInfo	localeInfo) {
		return	getAllDigits() > 0
				&& getFractionDigits() >= 0
				&& getAllDigits() >= getFractionDigits()
				&& (getChecker() == null
					|| getChecker() instanceof RangeChecker);
	}

	/** 機能.
	 * <P>
	 * 読み込むセルの型が数値型
	 * {@link CellTypeDefine#NUMBER}
	 * であることを確認する。
	 * </P>
	 * @param cellType
	 * セルの型の種別 {@link CellTypeDefine}
	 * @return
	 * 読み込むセルの型が数値型であれば true
	 * を、そうでなければ false を返す。
	 */
	@Override
	protected final boolean checkCellType(final CellTypeDefine cellType) {
		return	CellTypeDefine.NUMBER == cellType;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * 確認結果が不正であった場合それぞれのメッセージが出力される。
	 * @return
	 * 数値の定義域
	 * -1e(整数部桁数)≦入力値≦1e(整数部桁数)
	 * の範囲外であれば false
	 * を、そうでなければ true を返す。
	 */
	@Override
	protected final boolean isValidCellValue(
										final XlsHandler	xlsHandler,
										final StringBuilder	msgIdSb) {
		setRoundedValue(new BigDecimal(xlsHandler.getNumberCellValue())
								.setScale(getFractionDigits(),
											getRoundingMode()));
		boolean result
				= BigDecimal.TEN
							.pow(getAllDigits() - getFractionDigits())
							.compareTo(getRoundedValue().abs()) > 0;
		if (!result) {
			msgIdSb.append(getOutOfRangeMessageId());
		}
		else if (!(result = (getChecker() == null
							|| ((RangeChecker)getChecker())
													.check(xlsHandler))
							)) {
			msgIdSb.append(((RangeChecker)getChecker()).getMessage());
		}
		return result;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * {@link NumberCellDefine#isValidCellValue(XlsHandler, StringBuilder)}
	 * で求めた数値型セルの入力値を丸めた値
	 * {@link NumberCellDefine#roundedValue}
	 * を文字列にして返す。
	 */
	@Override
	protected final String convertCellValue(final XlsHandler xlsHandler) {
		return getRoundedValue().toString();
	}

}
