package readxls.type;
/*-**********************************************************************
 * File: StringCellDefine.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.io.UnsupportedEncodingException;
import java.util.regex.Pattern;

import readxls.ReadXlsException;
import readxls.ReturnCode;
import readxls.XlsHandler;
import readxls.control.CellDefine;
import readxls.control.LocaleInfo;

/** 読み込みセルの項目定義.
 * <P>
 * 文字列型のセルを読み込むための項目定義を行う。
 * <BR>
 * 指定する定義情報は以下のとおり。
 * </P>
 * <UL>
 * <LI>
 * セルを読み込む列の番号 (1 ～)
 * </LI>
 * <LI>
 * 項目名
 * </LI>
 * <LI>
 * 入力必須であるときに未入力だった場合のメッセージ
 * </LI>
 * <LI>
 * セルの型が文字列値でなかった場合のメッセージ
 * </LI>
 * <LI>
 * セルに入力された文字列を
 * {@link LocaleInfo#getCharsetToGetbytes()}
 * で取得できる文字コードで変換した結果の最大バイト数
 * </LI>
 * <LI>
 * 入力された文字列に使用できる文字種を示す正規表現
 * </LI>
 * <LI>
 * 最大文字数を超える長さの文字列が入力されていたことを意味するメッセージ
 * </LI>
 * <LI>
 * 使用できる文字意外が入力されていたことを意味するメッセージ
 * </LI>
 * </UL>
 * @see CellDefine
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class StringCellDefine extends CellDefine {

	/** フィールド.
	 * <P>
	 * 文字列型セルに入力された文字列
	 * </P>
	 */
	private String cellValue = null;

	/** フィールド.
	 * <P>
	 * セルに入力された文字列のバイト数を求めるために使用する
	 * 変換文字コード
	 * </P>
	 */
	private String charsetToGetbytes = null;

	/** フィールド.
	 * <P>
	 * 使用できる文字種
	 * {@link StringCellDefine#regex}
	 * に対応する
	 * {@link Pattern}
	 * </P>
	 */
	private Pattern pattern = null;

	/** 項目定義情報.
	 * <P>
	 * 文字列型セルに入力できる文字種を指定する正規表現
	 * </P>
	 */
	private String regex = null;

	/** 項目定義情報.
	 * <P>
	 * {@link StringCellDefine#charsetToGetbytes}
	 * で文字列型セルの入力値を文字コード変換した結果の最大バイト数
	 * </P>
	 */
	private int maxBytes = 0;

	/** 項目定義情報.
	 * <P>
	 * 文字列型セルの入力値が最大バイト数を超えていることを意味する
	 * メッセージ
	 * </P>
	 */
	private String overMaxBytesMessageId = null;

	/** 項目定義情報.
	 * <P>
	 * 文字列型セルに入力できる文字種意外の文字が使用されていることを示す
	 * メッセージ
	 * </P>
	 */
	private String illegalCharClassMessageId = null;

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#cellValue}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#cellValue}
	 * の値
	 */
	private String getCellValue() {
		return this.cellValue;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#cellValue}
	 * の setter 。
	 * @param cellValue
	 * {@link StringCellDefine#cellValue}
	 * に設定する値
	 */
	private void setCellValue(final String cellValue) {
		this.cellValue = cellValue;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#charsetToGetbytes}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#charsetToGetbytes}
	 * の値
	 */
	private String getCharsetToGetbytes() {
		return this.charsetToGetbytes;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#charsetToGetbytes}
	 * の setter 。
	 * @param charsetToGetbytes
	 * {@link StringCellDefine#charsetToGetbytes}
	 * に設定する値
	 */
	private void setCharsetToGetbytes(final String charsetToGetbytes) {
		this.charsetToGetbytes = charsetToGetbytes;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#pattern}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#pattern}
	 * の値
	 */
	private Pattern getPattern() {
		return this.pattern;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#pattern}
	 * の setter 。
	 * @param pattern
	 * {@link StringCellDefine#pattern}
	 * に設定する値
	 */
	private void setPattern(final Pattern pattern) {
		this.pattern = pattern;
	}

	/** ユーティリティ.
	 * <P>
	 * {@link StringCellDefine#regex}
	 * に指定された正規表現に対応する
	 * {@link Pattern}
	 * を、
	 * {@link StringCellDefine#pattern}
	 * に設定する。
	 * </P>
	 * <P>
	 * {@link StringCellDefine#regex}
	 * が null か空文字列 "" であれば、
	 * {@link StringCellDefine#pattern}
	 * に null を設定する。
	 * </P>
	 */
	private void setPattern() {
		String regex = getRegex();
		setPattern((regex != null && !"".equals(regex))
					?	Pattern.compile(regex)
					:	null);
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#regex}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#regex}
	 * の値
	 */
	public final String getRegex() {
		return this.regex;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#regex}
	 * の setter 。
	 * <P>
	 * {@link StringCellDefine#setPattern()}
	 * で対応する
	 * {@link Pattern}
	 * も設定する。
	 * </P>
	 * @param regex
	 * {@link StringCellDefine#regex}
	 * に設定する値
	 */
	public final void setRegex(final String regex) {
		this.regex = regex;
		setPattern();
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#maxBytes}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#maxBytes}
	 * の値
	 */
	public final int getMaxBytes() {
		return this.maxBytes;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#maxBytes}
	 * の setter 。
	 * @param maxBytes
	 * {@link StringCellDefine#maxBytes}
	 * に設定する値
	 */
	public final void setMaxBytes(final int maxBytes) {
		this.maxBytes = maxBytes;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#overMaxBytesMessageId}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#overMaxBytesMessageId}
	 * の値
	 */
	public final String getOverMaxBytesMessageId() {
		return this.overMaxBytesMessageId;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#overMaxBytesMessageId}
	 * の setter 。
	 * @param overMaxBytesMessageId
	 * {@link StringCellDefine#overMaxBytesMessageId}
	 * に設定する値
	 */
	public final void setOverMaxBytesMessageId(
									final String overMaxBytesMessageId) {
		this.overMaxBytesMessageId = overMaxBytesMessageId;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#illegalCharClassMessageId}
	 * の getter 。
	 * @return
	 * {@link StringCellDefine#illegalCharClassMessageId}
	 * の値
	 */
	public final String getIllegalCharClassMessageId() {
		return this.illegalCharClassMessageId;
	}

	/** getter/setter.
	 * <BR>
	 * {@link StringCellDefine#illegalCharClassMessageId}
	 * の setter 。
	 * @param illegalCharClassMessageId
	 * {@link StringCellDefine#illegalCharClassMessageId}
	 * に設定する値
	 */
	public final void setIllegalCharClassMessageId(
								final String illegalCharClassMessageId) {
		this.illegalCharClassMessageId = illegalCharClassMessageId;
	}

	/** コンストラクタ.
	 * <P>
	 * {@link BooleanCellDefine#valueOfTrue}
	 * と
	 * {@link BooleanCellDefine#valueOfFalse}
	 * の値も指定できる。
	 * </P>
	 * @param columnNo
	 * 読み込むセルの列番号 (1 ～)
	 * @param name
	 * 項目名
	 * @param lostMandatoryMessageID
	 * 入力必須である場合に未入力であることを示すメッセージ
	 * @param invalidTypeMessageId
	 * 列番号のセルが文字列型でなかったことを示すメッセージ
	 * @param maxBytes
	 * セルの入力値を
	 * {@link LocaleInfo#getCharsetToGetbytes()}
	 * で取得する文字コードで変換した結果の最大バイト数
	 * @param regex
	 * 入力文字列に使用できる文字種を指定する正規表現
	 * @param overMaxBytesMessageId
	 * makBytes を超えるバイト数の文字列が入力されたことを意味する
	 * メッセージ
	 * @param illegalCharClassMessageId
	 * 使用できない文字種の文字が入力されたことを意味するメッセージ
	 */
	public StringCellDefine(final int		columnNo,
							final String	name,
							final String	lostMandatoryMessageID,
							final String	invalidTypeMessageId,
							final int		maxBytes,
							final String	regex,
							final String	overMaxBytesMessageId,
							final String	illegalCharClassMessageId) {
		super(columnNo,
				name,
				lostMandatoryMessageID,
				invalidTypeMessageId,
				null);
		this.maxBytes					= maxBytes;
		this.regex						= regex;
		this.overMaxBytesMessageId		= overMaxBytesMessageId;
		this.illegalCharClassMessageId	= illegalCharClassMessageId;
		setPattern();
	}

	/** 機能.
	 * <P>
	 * 文字列型セルの最大バイト数がゼロより大きな値を設定されていることを
	 * 確認する。
	 * </P>
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param localeInfo
	 * ロケール設定の情報
	 * @return
	 * 文字列型セルの最大バイト数がゼロより大きな値を設定されていなければ
	 * false
	 * を、そうでなければ true を返す。
	 */
	@Override
	protected final boolean checkParameters(
										final XlsHandler	xlsHandler,
										final LocaleInfo	localeInfo) {
		setCharsetToGetbytes(localeInfo.getCharsetToGetbytes());
		return getMaxBytes() > 0;
	}

	/** 機能.
	 * <P>
	 * 読み込むセルの型が文字列型
	 * {@link CellTypeDefine#STRING}
	 * であることを確認する。
	 * </P>
	 * @param cellType
	 * セルの型の種別 {@link CellTypeDefine}
	 * @return
	 * 読み込むセルの型が文字列型であれば true
	 * を、そうでなければ false を返す。
	 */
	@Override
	protected final boolean checkCellType(final CellTypeDefine cellType) {
		return	CellTypeDefine.STRING == cellType;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @param msgIdSb
	 * 確認結果が不正であった場合それぞれのメッセージが出力される。
	 * @return
	 * 入力された文字列が最大バイト数を超えているか、使用できる文字種意外
	 * の文字を使用している場合には false
	 * を、そうでなければ true を返す。
	 * @exception ReadXlsException
	 * {@link LocaleInfo#getCharsetToGetbytes()}
	 * で取得できる文字コードが Java でサポートされていない場合に発生する。
	 *
	 */
	@Override
	protected final boolean isValidCellValue(
										final XlsHandler	xlsHandler,
										final StringBuilder	msgIdSb)
												throws ReadXlsException {
		boolean result = true;
		try {
			setCellValue(xlsHandler.getStringCellValue());
			if (getCellValue() != null
			&& getMaxBytes()
					< getCellValue().getBytes(getCharsetToGetbytes())
									.length) {
				msgIdSb.append(getOverMaxBytesMessageId());
				result = false;
			}
			else if (getCellValue() == null
			|| (getPattern() != null
				&& !getPattern().matcher(getCellValue()).find())
			) {
				msgIdSb.append(getIllegalCharClassMessageId());
				result = false;
			}
		} catch (UnsupportedEncodingException ueex) {
			throw new ReadXlsException(ReturnCode.UNSUPPORTED_CHARSET);
		}
		return result;
	}

	/** 機能.
	 * @param xlsHandler
	 * Excel ファイルを読み込むための依存ライブラリ呼び出し手順の実装
	 * @return
	 * {@link StringCellDefine#isValidCellValue(XlsHandler, StringBuilder)}
	 * で設定された
	 * {@link StringCellDefine#cellValue}
	 * の値を返す。
	 */
	@Override
	protected final String convertCellValue(final XlsHandler xlsHandler) {
		return getCellValue();
	}

}
