package readxlsforjxl;
/*-**********************************************************************
 * File: XlsHandleForJxl.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.TimeZone;

import jxl.BooleanCell;
import jxl.Cell;
import jxl.CellType;
import jxl.DateCell;
import jxl.LabelCell;
import jxl.NumberCell;
import jxl.Sheet;
import jxl.Workbook;
import jxl.WorkbookSettings;
import jxl.read.biff.BiffException;
import readxls.ReadXlsException;
import readxls.ReturnCode;
import readxls.XlsHandler;
import readxls.control.LocaleInfo;
import readxls.type.CellTypeDefine;

/** Excel ファイル読み込み.
 * <P>
 * JExcelAPI を使用して Excel ファイルを読み込むための、
 * {@link XlsHandler}
 * インタフェースの実装クラス。
 * </P>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class XlsHandleForJxl implements XlsHandler {

	/** リソース.
	 * <P>
	 * Excel ファイルを読み込んだワークブック。
	 * </P>
	 */
	private Workbook wb = null;

	/** リソース.
	 * <P>
	 * {@link XlsHandleForJxl#wb}
	 * から取得したシート。
	 * </P>
	 */
	private Sheet sh = null;

	/** リソース.
	 * <P>
	 * 入力値を取得するためにシートのある行のセルをすべて保持する配列。
	 * </P>
	 */
	private Cell[] lastReadCells = new Cell[0];

	/** リソース.
	 * <P>
	 * {@link XlsHandleForJxl#lastReadCells}
	 * のある列のセルを保持する。
	 * </P>
	 */
	private Cell lastReadCell = null;

	/** リソース.
	 * <P>
	 * 読み込みを停止する行を判定するために、
	 * ある行に入力されているセルをすべて保持する配列。
	 * </P>
	 */
	private Cell[] breakerCells = null;

	/** リソース.
	 * <P>
	 * {@link XlsHandleForJxl#breakerCells}
	 * からある列のセルを取得して保持する。
	 * </P>
	 */
	private Cell breakerCell = null;

	/** 属性.
	 * <P>
	 * 日付型セルの入力値をタイムゾーンに補正するための補正値。
	 * </P>
	 */
	private int timeZoneRawOffset = 0;

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForJxl#wb}
	 * を返す。
	 */
	private Workbook getWb() {
		return this.wb;
	}

	/** getter/setter.
	 * @param wb
	 * {@link XlsHandleForJxl#wb}
	 * に設定するワークブック。
	 */
	private void setWb(final Workbook wb) {
		this.wb = wb;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForJxl#sh}
	 * を返す。
	 */
	private Sheet getSh() {
		return this.sh;
	}

	/** getter/setter.
	 * @param sh
	 * {@link XlsHandleForJxl#sh}
	 * に設定するシート。
	 */
	private void setSh(final Sheet sh) {
		this.sh = sh;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForJxl#lastReadCells}
	 * を返す。
	 */
	private Cell[] getLastReadCells() {
		return this.lastReadCells;
	}

	/** getter/setter.
	 * @param lastReadCells
	 * {@link XlsHandleForJxl#lastReadCells}
	 * に設定するセル配列
	 */
	private void setLastReadCells(final Cell[] lastReadCells) {
		this.lastReadCells = lastReadCells;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForJxl#lastReadCell}
	 * を返す。
	 */
	private Cell getLastReadCell() {
		return this.lastReadCell;
	}

	/** getter/setter.
	 * @param lastReadCell
	 * {@link XlsHandleForJxl#lastReadCell}
	 * に設定するセル
	 */
	private void setLastReadCell(final Cell lastReadCell) {
		this.lastReadCell = lastReadCell;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForJxl#breakerCells}
	 * を返す。
	 */
	private Cell[] getBreakerCells() {
		return this.breakerCells;
	}

	/** getter/setter.
	 * @param breakCells
	 * {@link XlsHandleForJxl#breakerCells}
	 * に設定するセル配列
	 */
	private void setBreakerCells(final Cell[] breakCells) {
		this.breakerCells = breakCells;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForJxl#breakerCell}
	 * を返す。
	 */
	private Cell getBreakerCell() {
		return breakerCell;
	}

	/** getter/setter.
	 * @param breakerCell
	 * {@link XlsHandleForJxl#breakerCell}
	 * に設定するセル
	 */
	private void setBreakerCell(final Cell breakerCell) {
		this.breakerCell = breakerCell;
	}

	/** 機能.
	 * @return
	 * {@link XlsHandleForJxl#timeZoneRawOffset}
	 * を返す。
	 */
	private int getTimeZoneRawOffset() {
		return this.timeZoneRawOffset;
	}

	/** 機能.
	 * <P>
	 * ロケール情報のタイムゾーン
	 * {@link LocaleInfo#getTimeZoneId()}
	 * の GMT からのオフセット値を求めて
	 * {@link XlsHandleForJxl#timeZoneRawOffset}
	 * に設定する。
	 * </P>
	 * @param localeInfo
	 * ロケール情報を設定する。
	 */
	@Override
	public final void setTimeZoneRawOffset(final LocaleInfo localeInfo) {
		this.timeZoneRawOffset
					= TimeZone.getTimeZone(localeInfo.getTimeZoneId())
								.getRawOffset();
	}

	/** 機能.
	 * @return
	 * 日付型セルの入力値 (GMT) を、
	 * {@link XlsHandleForJxl#setTimeZoneRawOffset(LocaleInfo)}
	 * で求めたオフセット値で補正したエポック値を返す。
	 * @param value
	 * 日付型セルの入力値
	 */
	@Override
	public final long adjustToTimeZone(final Date value) {
		return value.getTime() - getTimeZoneRawOffset();
	}

	@Override
	public final boolean isLoadedWorkbook() {
		return getWb() != null;
	}

	@Override
	public final boolean loadWorkbook(final InputStream		is,
										final LocaleInfo	localeInfo,
										final boolean		isOOXML)
									throws ReadXlsException, IOException {
		if (isOOXML) {
			throw new ReadXlsException(ReturnCode.NOT_EXL_FILE);
		}
		try {
			WorkbookSettings wbs = new WorkbookSettings();
			wbs.setEncoding(localeInfo.getExcodeOfExlWorkbook());
			wbs.setGCDisabled(true);
			setWb(Workbook.getWorkbook(is, wbs));
		} catch (BiffException bex) {
			throw new ReadXlsException(ReturnCode.NOT_EXL_FILE);
		}
		return isLoadedWorkbook();
	}

	@Override
	public final boolean setSheet(final String sheetName) {
		setSh(getWb().getSheet(sheetName));
		return getSh() != null;
	}

	@Override
	public final int getLastRowNum() {
		return getSh().getRows();
	}

	@Override
	public final void readInCells(final int rowNo) {
		setLastReadCells((rowNo > 0)
							?	getSh().getRow(rowNo - 1)
							:	new Cell[0]);
	}

	@Override
	public final int lengthOfLastReadCells() {
		return getLastReadCells().length;
	}

	@Override
	public final void readInLastReadCell(final int columnNo) {
		setLastReadCell((columnNo > 0)
						?	getLastReadCells()[columnNo - 1]
						:	null);
	}

	/** ユーティリティ.
	 * <P>
	 * 指定されたセルの型を求める。
	 * </P>
	 * @param cell
	 * 型を求めるセル
	 * @return
	 * セルの型
	 * @see CellTypeDefine
	 */
	private CellTypeDefine getCellType(final Cell cell) {
		CellTypeDefine result = CellTypeDefine.EMPTY;
		CellType cellType = (cell != null)
							?	cell.getType()
							:	CellType.EMPTY;
		if (CellType.BOOLEAN_FORMULA == cellType
		|| CellType.BOOLEAN == cellType) {
			result = CellTypeDefine.BOOLEAN;
		}
		else if (CellType.STRING_FORMULA == cellType
		|| CellType.LABEL == cellType) {
			result = CellTypeDefine.STRING;
		}
		else if (CellType.DATE_FORMULA == cellType
		|| CellType.DATE == cellType) {
			result = CellTypeDefine.DATE;
		}
		else if (CellType.NUMBER_FORMULA == cellType
		|| CellType.NUMBER == cellType) {
			result = CellTypeDefine.NUMBER;
		}
		return result;
	}

	@Override
	public final CellTypeDefine getCellType() {
		return getCellType(getLastReadCell());
	}

	@Override
	public final boolean getBooleanCellValue() {
		return ((BooleanCell)getLastReadCell()).getValue();
	}

	@Override
	public final Date getDateCellValue() {
		return ((DateCell)getLastReadCell()).getDate();
	}

	@Override
	public final double getNumberCellValue() {
		return ((NumberCell)getLastReadCell()).getValue();
	}

	@Override
	public final String getStringCellValue() {
		return ((LabelCell)getLastReadCell()).getContents();
	}

	@Override
	public final void readInBreakerCells(final int rowNo) {
		setBreakerCells((rowNo > 0)
						?	getSh().getRow(rowNo - 1)
						:	new Cell[0]);
	}

	@Override
	public final int getBreakerCellsLength() {
		return getBreakerCells().length;
	}

	@Override
	public final void readInBreakerCell(final int columnNo) {
		setBreakerCell((columnNo <= getBreakerCells().length)
						?	getBreakerCells()[columnNo - 1]
						:	null);
	}

	@Override
	public final boolean isBreakerCellEmpty() {
		return	getBreakerCell() == null
				|| getCellType(getBreakerCell()) == CellTypeDefine.EMPTY;
	}

	@Override
	public final void close() {
		if (isLoadedWorkbook()) {
			getWb().close();
			setWb(null);
			setSh(null);
			setLastReadCells(null);
			setLastReadCell(null);
			setBreakerCells(null);
			setBreakerCell(null);
		}
	}

}
