package readxlsforpoi;
/*-**********************************************************************
 * File: XlsHandleForPoi.java
 * 	(Extracted from
 *	   "$Id$")
 *
 * Abstracts:
 *  This Java source code implements "readxls" package.
 *  "readxls" package reads a excel file (extension ".xls" only),
 *  and create a hashtable contains header cell values or
 *  a list of detaile record cell values.
 *  If package find several invalid data cells. information about
 *  the invalid cells is outputed into a list.
 *  This package depends on JExcelAPI.
 *
 * Copyright (C) 2014 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *-*********************************************************************/

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.DateUtil;
import org.apache.poi.ss.usermodel.FormulaEvaluator;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;

import readxls.ReadXlsException;
import readxls.ReturnCode;
import readxls.XlsHandler;
import readxls.control.LocaleInfo;
import readxls.type.CellTypeDefine;

/** Excel ファイル読み込み.
 * <P>
 * Apatche POI を使用して Excel ファイルを読み込むための、
 * {@link XlsHandler}
 * インタフェースの実装クラス。
 * </P>
 *
 * @author maoyam (maoyam@mail.goo.ne.jp)
 */
public class XlsHandleForPoi implements XlsHandler {

	/** リソース.
	 * <P>
	 * Excel ファイルを読み込んだワークブック。
	 * </P>
	 */
	private Workbook wb = null;

	/** リソース.
	 * <P>
	 * {@link XlsHandleForPoi#wb}
	 * から取得したシート。
	 * </P>
	 */
	private Sheet sh = null;

	/** リソース.
	 * <P>
	 * 入力値を取得するためにシートのある行のセルをすべて保持する配列。
	 * </P>
	 */
	private Cell[] lastReadCells = new Cell[0];

	/** リソース.
	 * <P>
	 * {@link XlsHandleForPoi#lastReadCells}
	 * のある列のセルを保持する。
	 * </P>
	 */
	private Cell lastReadCell = null;

	/** リソース.
	 * <P>
	 * 読み込みを停止する行を判定するために、
	 * ある行に入力されているセルをすべて保持する配列。
	 * </P>
	 */
	private Cell[] breakerCells = null;

	/** リソース.
	 * <P>
	 * {@link XlsHandleForPoi#breakerCells}
	 * からある列のセルを取得して保持する。
	 * </P>
	 */
	private Cell breakerCell = null;

	/** 属性.
	 * <P>
	 * 日付型セルの入力値をタイムゾーンに補正するための補正値。
	 * </P>
	 */
	private int timeZoneRawOffset = 0;

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForPoi#wb}
	 * を返す。
	 */
	private Workbook getWb() {
		return this.wb;
	}

	/** getter/setter.
	 * @param wb
	 * {@link XlsHandleForPoi#wb}
	 * に設定するワークブック。
	 */
	private void setWb(final Workbook wb) {
		this.wb = wb;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForPoi#sh}
	 * を返す。
	 */
	private Sheet getSh() {
		return this.sh;
	}

	/** getter/setter.
	 * @param sh
	 * {@link XlsHandleForPoi#sh}
	 * に設定するシート。
	 */
	private void setSh(final Sheet sh) {
		this.sh = sh;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForPoi#lastReadCells}
	 * を返す。
	 */
	private Cell[] getLastReadCells() {
		return this.lastReadCells;
	}

	/** getter/setter.
	 * @param lastReadCells
	 * {@link XlsHandleForPoi#lastReadCells}
	 * に設定するセル配列
	 */
	private void setLastReadCells(final Cell[] lastReadCells) {
		this.lastReadCells = lastReadCells;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForPoi#lastReadCell}
	 * を返す。
	 */
	private Cell getLastReadCell() {
		return this.lastReadCell;
	}

	/** getter/setter.
	 * @param lastReadCell
	 * {@link XlsHandleForPoi#lastReadCell}
	 * に設定するセル
	 */
	private void setLastReadCell(final Cell lastReadCell) {
		this.lastReadCell = lastReadCell;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForPoi#breakerCells}
	 * を返す。
	 */
	private Cell[] getBreakerCells() {
		return this.breakerCells;
	}

	/** getter/setter.
	 * @param breakCells
	 * {@link XlsHandleForPoi#breakerCells}
	 * に設定するセル配列
	 */
	private void setBreakerCells(final Cell[] breakCells) {
		this.breakerCells = breakCells;
	}

	/** getter/setter.
	 * @return
	 * {@link XlsHandleForPoi#breakerCell}
	 * を返す。
	 */
	private Cell getBreakerCell() {
		return breakerCell;
	}

	/** getter/setter.
	 * @param breakerCell
	 * {@link XlsHandleForPoi#breakerCell}
	 * に設定するセル
	 */
	private void setBreakerCell(final Cell breakerCell) {
		this.breakerCell = breakerCell;
	}

	/** 機能.
	 * @return
	 * {@link XlsHandleForPoi#timeZoneRawOffset}
	 * を返す。
	 */
	private int getTimeZoneRawOffset() {
		return this.timeZoneRawOffset;
	}

	/** 機能.
	 * <P>
	 * ロケール情報のタイムゾーン
	 * {@link LocaleInfo#getTimeZoneId()}
	 * の GMT からのオフセット値を求めて
	 * {@link XlsHandleForPoi#timeZoneRawOffset}
	 * に設定する。
	 * </P>
	 * @param localeInfo
	 * ロケール情報を設定する。
	 */
	@Override
	public final void setTimeZoneRawOffset(final LocaleInfo localeInfo) {
		this.timeZoneRawOffset = 0;
	}

	/** 機能.
	 * @param value
	 * 日付型セルの入力値
	 * @return
	 * 日付型セルの入力値 (GMT) を、
	 * {@link XlsHandleForPoi#setTimeZoneRawOffset(LocaleInfo)}
	 * で求めたオフセット値で補正したエポック値を返す。
	 */
	@Override
	public final long adjustToTimeZone(final Date value) {
		return value.getTime() + getTimeZoneRawOffset();
	}

	@Override
	public final boolean isLoadedWorkbook() {
		return getWb() != null;
	}

	@Override
	public final boolean loadWorkbook(final InputStream		is,
										final LocaleInfo	localeInfo,
										final boolean		isOOXML)
									throws IOException, ReadXlsException {
		try {
			setWb(WorkbookFactory.create(is));
		} catch (InvalidFormatException ifex) {
			throw new ReadXlsException(ReturnCode.NOT_EXL_FILE);
		}
		return isLoadedWorkbook();
	}

	@Override
	public final boolean setSheet(final String sheetName) {
		setSh(getWb().getSheet(sheetName));
		return getSh() != null;
	}

	@Override
	public final int getLastRowNum() {
		return getSh().getLastRowNum() + 1;
	}

	/** ユーティリティ.
	 * <P>
	 * 指定された行番号 (1～) のセルを読み込む。
	 * </P>
	 * @param rowNo
	 * 行番号 (1～)
	 * @return
	 * 読み込んだセルの配列
	 */
	private Cell[] readCells(final int rowNo) {
		Cell[] cells = null;
		if (rowNo > 0) {
			Row row = getSh().getRow(rowNo - 1);
			int lastCellNum = (row != null && row.getLastCellNum() > -1)
								?	row.getLastCellNum()
								:	0;
			cells = new Cell[lastCellNum];
			for (int c = 0; c < lastCellNum; ++c) {
				cells[c] = row.getCell(c);
			}
		}
		else {
			cells = new Cell[0];
		}
		return cells;
	}

	@Override
	public final void readInCells(final int rowNo) {
		setLastReadCells(readCells(rowNo));
	}

	@Override
	public final int lengthOfLastReadCells() {
		return getLastReadCells().length;
	}

	@Override
	public final void readInLastReadCell(final int columnNo) {
		setLastReadCell((columnNo > 0)
				?	getLastReadCells()[columnNo - 1]
				:	null);
	}

	/** ユーティリティ.
	 * <P>
	 * セルに設定されている式を評価した結果のセルを取得する。
	 * </P>
	 * @param cell
	 * 式を設定されているセル
	 * @return
	 * 式を評価した結果のセル
	 */
	private Cell evaluateCell(final Cell cell) {
		FormulaEvaluator evaluator
				= getWb().getCreationHelper()
							.createFormulaEvaluator();
		return evaluator.evaluateInCell(cell);
	}

	/** ユーティリティ.
	 * <P>
	 * 指定されたセルの型を求める。
	 * </P>
	 * @param cell
	 * 型を求めるセル
	 * @return
	 * セルの型
	 * @see CellTypeDefine
	 */
	private CellTypeDefine getCellType(final Cell cell) {
		CellTypeDefine result = CellTypeDefine.EMPTY;
		Cell theCell = cell;
		int cellType = (theCell != null)
						?	theCell.getCellType()
						:	Cell.CELL_TYPE_BLANK;
		if (Cell.CELL_TYPE_FORMULA == cellType) {
			theCell = evaluateCell(theCell);
			cellType = theCell.getCellType();
		}
		if (Cell.CELL_TYPE_BOOLEAN == cellType) {
			result = CellTypeDefine.BOOLEAN;
		}
		else if (Cell.CELL_TYPE_STRING == cellType) {
			result = CellTypeDefine.STRING;
		}
		else if (Cell.CELL_TYPE_NUMERIC == cellType) {
			if (DateUtil.isCellDateFormatted(theCell)) {
				result = CellTypeDefine.DATE;
			}
			else {
				result = CellTypeDefine.NUMBER;
			}
		}
		return result;
	}

	@Override
	public final CellTypeDefine getCellType() {
		return getCellType(getLastReadCell());
	}

	@Override
	public final boolean getBooleanCellValue() {
		Cell cell = getLastReadCell();
		if (Cell.CELL_TYPE_FORMULA == cell.getCellType()) {
			cell = evaluateCell(cell);
		}
		return cell.getBooleanCellValue();
	}

	@Override
	public final Date getDateCellValue() {
		Cell cell = getLastReadCell();
		if (Cell.CELL_TYPE_FORMULA == cell.getCellType()) {
			cell = evaluateCell(cell);
		}
		return cell.getDateCellValue();
	}

	@Override
	public final double getNumberCellValue() {
		Cell cell = getLastReadCell();
		if (Cell.CELL_TYPE_FORMULA == cell.getCellType()) {
			cell = evaluateCell(cell);
		}
		return cell.getNumericCellValue();
	}

	@Override
	public final String getStringCellValue() {
		Cell cell = this.lastReadCell;
		if (Cell.CELL_TYPE_FORMULA == cell.getCellType()) {
			cell = evaluateCell(cell);
		}
		return cell.getStringCellValue();
	}

	@Override
	public final void readInBreakerCells(final int rowNo) {
		setBreakerCells(readCells(rowNo));
	}

	@Override
	public final int getBreakerCellsLength() {
		return getBreakerCells().length;
	}

	@Override
	public final void readInBreakerCell(final int columnNo) {
		setBreakerCell((columnNo <= getBreakerCells().length)
				?	getBreakerCells()[columnNo - 1]
				:	null);
	}

	@Override
	public final boolean isBreakerCellEmpty() {
		return	getBreakerCell() == null
				|| getCellType(getBreakerCell()) == CellTypeDefine.EMPTY;
	}

	@Override
	public final void close() {
		if (isLoadedWorkbook()) {
			setWb(null);
			setSh(null);
			setLastReadCells(null);
			setLastReadCell(null);
			setBreakerCells(null);
			setBreakerCell(null);
		}
	}

}
